package net.quartzware;

import java.awt.*;
import javax.swing.*;
import java.lang.*;
import java.awt.event.*;
import java.util.*;

/**
 AddKeyDlg
 Dialog box that allows user to select what type of key to add to the resource bundle.
 Dialog has two fields: for the user to enter the key name, and to select the type of key.
 Allows the user to select which key type he/she would like to add.
 @author Chris Perluss
 @version 0.7.1
*/

// Version History
//     11/30/00 - Version 0.9   Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//     11/29/00 - Version 0.7.1 Susan Hunton
//                Modified adt header, added pseudocode
//              - Anthony Tomarchio - Internationalized in English and Italian
//     11/19/00 - Version 0.5 Chris Perluss
//                formatted for style
//     11/17/00 - Version 0.1 Anthony Tomarchio
//                Created using WebGain Visual Cafe 4.1

public class AddKeyDlg extends CenterDlg {
    ResBundle resBundle = new ResBundle("", null, false);
    JLabel keyLabel = new JLabel();
    JLabel entryLabel = new JLabel();
    JTextField keyField = new JTextField("button1");
    JComboBox entryComboBox = new JComboBox(new String[]{"Binary File",
        "Binary Number", "Integer", "Integer Array", "Message", "Object",
        "String", "String Array", "Tagged Array"});

    Class[] ValueClasses = new Class[]{BinaryPath.class, BinaryVal.class,
        IntegerVal.class, IntArray.class, Message.class, ObjectVal.class,
        StringVal.class, StringArray.class, TaggedArray.class};

    JButton okButton = new JButton();
    JButton cancelButton = new JButton();

    Value val;
    String key;

    //Internationalization variables
    Locale currentLocale = Locale.getDefault();
    ResourceBundle bundle = ResourceBundle.getBundle("RBEdit", currentLocale);

    /**
     Constructs the AddKeyDlg and sets the title equal to sTitle
     PRE:  sTitle cannot be null
     POST:  dialog is created
    */
    public AddKeyDlg(String sTitle) {
        this();
        setTitle(sTitle);
    }

    /**
     Constructs the AddKeyDlg
     PRE:  none
     POST:  dialog is created
    */
    public AddKeyDlg() {
        this((Frame)null);
    }
    /**
     Constructs the AddKeyDlg
     PRE:  none
     POST:  dialog is created
    */
    public AddKeyDlg(ResBundle rb) {
        this((Frame)null);
        resBundle = rb;
    }

    /**
     Constructs the AddKeyDlg
     PRE:  parent cannot be null
     POST:  dialog is created
    */
    public AddKeyDlg(Frame parent) {
        //super(parent, "Add Key");
        super(parent);
        setTitle(bundle.getString("addKeyDlg"));

        doTheLayout();
        pack();
    }

    /**
     Sets the layout of the dialog
     PRE:  none
     POST:  layout is complete and all buttons, fields ect are in their proper position
    */
    private void doTheLayout() {
        setResizable(false);
        setModal(true);
        getContentPane().setLayout(null);

        //Create key label
        keyLabel.setText(bundle.getString("keyLabel"));
        keyLabel.setForeground(java.awt.Color.black);
        keyLabel.setBounds(12,12,75,25);
        getContentPane().add(keyLabel);

        //Create entry type label
        entryLabel.setText(bundle.getString("entryLabel"));
        entryLabel.setForeground(java.awt.Color.black);
        entryLabel.setBounds(12,48,75,25);
        getContentPane().add(entryLabel);

        // create field for key name
        keyField.setBounds(84,12,150,25);
        getContentPane().add(keyField);

        // create entry combo box for entry type selection
        entryComboBox.setMaximumRowCount(15);
        entryComboBox.setBounds(84,48,150,25);
        entryComboBox.setSelectedIndex(6);
        getContentPane().add(entryComboBox);

        // create ok button
        okButton.setText(bundle.getString("okButton"));
        okButton.setMnemonic((int)'O');
        okButton.setBounds(12,96,100,25);
        getContentPane().add(okButton);

        // create cancel button
        cancelButton.setText(bundle.getString("cancelButton"));
        cancelButton.setMnemonic((int)'C');
        cancelButton.setBounds(132,96,100,25);
        getContentPane().add(cancelButton);

        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                key = keyField.getText();
                if (key.equals("")) {
                    JOptionPane.showMessageDialog(AddKeyDlg.this,
                            bundle.getString("enterKeyName"),
                            bundle.getString("error"),
                            JOptionPane.ERROR_MESSAGE);

                    return;
                }

                Vector entries = resBundle.getAllEntries();
                for (int idx = 0; idx < entries.size(); idx++) {
                    if (((Entry)entries.elementAt(idx)).getKey().equals(key)) {
                        JOptionPane.showMessageDialog(AddKeyDlg.this,
                            bundle.getString("keyExists"), bundle.getString("error"),
                            JOptionPane.ERROR_MESSAGE);

                        return;
                    }
                }

                try {
                    val = (Value)ValueClasses[
                            entryComboBox.getSelectedIndex()].newInstance();
                    setVisible(false);
                } catch (Exception ex) {ex.printStackTrace();}
            }
        });

        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });
    }

    /**
     Sets the preferred size of the window and returns is as a Dimension
     PRE: None
     POST: Dimension returned
    */
    public Dimension getPreferredSize() {
        return new Dimension(245,160);
    }

    /**
     Gets the instantiation of the Value of the type the user selected.
     PRE: None
     POST: none
    */
    public Value getValue() {
        return val;
    }

    /**
     Gets the Key Name the user typed in
     PRE: none
     POST: none
    */
    public String getKeyName() {
        return key;
    }

    /**
     Makes dialog visible to user
    */
    public void setVisible(boolean vis) {
        if (vis) {
            setLocation(50, 50);
        }
        super.setVisible(vis);
    }
}