/* Generated by Together */

package net.quartzware;

import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

/**
 EditorWindow
 A window which displays the contents of a Resource Bundle and allows a
 user to edit the contents of a Resource Bundle.  Default version of this
 window has 4 columns: key, value, parent value, comments,
 @author Ben Leinweber
 @version 0.7.1
*/

// Version History
//     02/25/01 - Version 1.1   Tony Johansen
//                Changed calls to DeleteKeyDlg.java to doRemoveKey
//                in the MainWindow.java
//     01/30/01 - Version 1.0   Tony Johansen
//                Added internalFrameListener
//     01/29/01 - Version 0.95 Albert Lee Implemented Edit String
//                Functionality
//     11/30/00 - Version 0.9   Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//     11/29/00 - version 0.7.1 - Susan Hunton
//                modified adt header, checked pseudocode
//              - Anthony Tomarchio - Internationalized RBEditTableModel
//     11/28/00 - Chris Perluss
//                Added toolbar to EditorWindow
//     11/20/00 - Version 0.6 - Anthony Tomarchio
//                Updated for internationalization
//                Draws in the MainWindow now - Chris Perluss
//     11/16/00 - Ver 0.3 Tony Johansen
//                Added Pre and Post Conditions
//     11/15/00 - Ver 0.2 Chris Perlus
//                Added algorithm pseudocode
//     11/01/00 - Ver 0.1 Ben Leinweber
//                Initial Creation.  Javadoc code added.


public class EditorWindow extends InternalWindow {

    final int valCol = 1;
    final int comCol = 3;

    ImageIcon addKeyIcon;
    ImageIcon remKeyIcon;
    ImageIcon editValIcon;
    ImageIcon editComIcon;

    JPanel toolbarPanel = new JPanel();
    JToolBar toolbar = new JToolBar();
    JPopupMenu popup = new JPopupMenu();

    JButton addKeyButton = new JButton();
    JButton remKeyButton = new JButton();
    JButton editValButton = new JButton();
    JButton editComButton = new JButton();

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     */
    JMenuItem menuItem;
    ResBundle resBundle;
    JPopupMenu popupMenu;
    JTable table;
    RBEditTableModel model;
    RBEditTableCellRenderer renderer;
    RBEditTableCellEditor editor;

    /**
     PopupListener
     This class listens for right clicks, then pops up the Context menu for the
     editor window.
    */
    public class PopupListener extends MouseAdapter {
        /**
         Called by the java framework when a  mouse button has been pressed in
         the Editor Window.

         PRE: none
         POST: popup menu may be displayed
        */
        public void mousePressed(MouseEvent e) {
            maybeShowPopup(e);
        }

        /**
         Called by the java framework when a  mouse button has been released in
         the Editor Window.

         PRE: none
         POST: popup menu may be displayed
        */
        public void mouseReleased(MouseEvent e) {
            maybeShowPopup(e);
        }

        /**
         Displays the popup menu if the trigger button was pressed.

         PRE: none
         POST: popup menu may be displayed
        */
        private void maybeShowPopup(MouseEvent e) {
            if (e.isPopupTrigger()) {
                popup.show(e.getComponent(), e.getX(), e.getY());
            }
        }
    }


    /**
     RBEditTableCellEditor
     Custom table editor used to properly edit the different Value types.
    */
    public class RBEditTableCellEditor extends DefaultCellEditor {
        /**
         Default Constructor

         PRE: none
         POST: text field is default editing component
        */
        public RBEditTableCellEditor() {
            super(new JTextField());
        }

        public Component getTableCellEditorComponent(JTable table, Object value,
                boolean isSelected, int row, int column) {

            resBundle.setDirty(true);
            Entry ent = resBundle.getEntry(row);
            ent.setUnique(true);

            if (column == 3 || ((Value)value).editValue()) {
                return super.getTableCellEditorComponent(table, value,
                    isSelected, row, column);
            } else {
                cancelCellEditing();
                resBundle.fireEntryChanged(ent, row);
                return null;
            }
        }
    }


    /**
     RBEditTableCellRenderer
     Custom table renderer used to properly display the different Value types.
    */
    public class RBEditTableCellRenderer extends DefaultTableCellRenderer {
        public Component getTableCellRendererComponent(JTable table, Object key,
                boolean isSel, boolean hasFocus, int row, int column) {

            // GET value to be edited.
            Value val = (Value)model.getValueAt(row, 1);

            // INIT label to draw.
            JLabel label = null;

            // IF column is 0 THEN
            //     CREATE a label which uses an icon
            // ELSE
            //     CREATE a label without an icon
            if (column == 0) {
                label = new JLabel("" + key, val.getIcon(), LEFT);
            } else {
                label = new JLabel("" + key, LEFT);
            }

            // make label transparent so the window background color shows through
            label.setOpaque(true);

            // set the background and foreground color of the label based on
            // whether it is selected or not.
            if (isSel) {
                label.setBackground(new Color(0, 0, 128));
                label.setForeground(Color.white);
            } else {
                label.setBackground(Color.white);
                label.setForeground(Color.black);
            }

            // return the label
            return label;
        }
    }


    /**
     RBEditTableModel
     An interface that allows a JTable to directly access our data, rather than
     arranging our data in a generic form that the DefaultTableModel could
     digest.
    */
    public class RBEditTableModel extends AbstractTableModel
            implements RBListener {

        //Internationalization variables
        Locale currentLocale = Locale.getDefault();
        ResourceBundle bundle = ResourceBundle.getBundle("RBEdit", currentLocale);

        private String[] columnNames = {bundle.getString("editorWindowKeyColumn"),
                                        bundle.getString("editorWindowValueColumn"),
                                        bundle.getString("editorWindowParentColumn"),
                                        bundle.getString("editorWindowCommentsColumn")};

        public Class getColumnClass(int col) {
            //IF col is 1 or col is 2 THEN
            //    return the reflection Class for Value
            //ELSE IF col is 0 or col is 3 THEN
            //    return the reflection Class of a string
            //ELSE
            //    return the reflection Class of Object

            if (col == 1 || col == 2) {
                return Value.class;
            } else if (col == 0 || col == 3) {
                return String.class;
            } else {
                return Object.class;
            }
        }

        public int getColumnCount() {
            return 4;
        }

        public String getColumnName(int col) {
            return columnNames[col];
        }

        public int getRowCount() {
            return resBundle.getAllEntries().size();
        }

        public Object getValueAt(int row, int col) {
            return resBundle.getEntry(row).elementAt(resBundle, col);
        }

        public boolean isCellEditable(int row, int col) {
            return (col == 1 || col == 3);
        }

        public void setValueAt(Object aValue, int row, int col) {
            if (col == 1) {
                resBundle.getEntry(row).setValue((String)aValue);
            } else if (col == 3) {
                resBundle.getEntry(row).setComment((String)aValue);
            }
            resBundle.fireEntryChanged(resBundle.getEntry(row), row);
        }

        public void entryAdded(RBEvent rbe) {
            fireTableRowsInserted(resBundle.getAllEntries().size()-1,
                resBundle.getAllEntries().size()-1);
        }

        public void entryInserted(RBEvent rbe) {
            fireTableRowsInserted(rbe.getIndex(), rbe.getIndex());
        }

        public void entryRemoved(RBEvent rbe) {
            fireTableRowsDeleted(rbe.getIndex(), rbe.getIndex());
        }

        public void entryChanged(RBEvent rbe) {
            fireTableRowsUpdated(rbe.getIndex(), rbe.getIndex());
        }
    }


    //Internationalization variables
    Locale currentLocale = Locale.getDefault();
    ResourceBundle bundle = ResourceBundle.getBundle("RBEdit", currentLocale);

    /**
     Constructs an Editor Window that will display the given ResBundle.
     @param rb A resource bundle that the Editor Window can Display and Edit

     PRE:  rb does not equal null.
     POST: class is created and GUI components have been added and arranged
           appropriately.
    */
    public EditorWindow(ResBundle rb) {
        super("", true, true, true, true);

        if (rb.isLooseFile()) {
			setTitle(rb.getFile().getPath());
        } else if (rb.getSuffix().equals("")) {
            setTitle("root");
        } else {
            setTitle(rb.getSuffix());
        }

        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        doTheLayout(rb);
        doPopupMenu();
        doAddListeners();

        pack();
        setVisible(true);
    }

    /**
     Lays out this window

     PRE: this class's attributes have been set
     POST: GUI components have been added to the window and arranged appropriately.
    */
    public void doTheLayout(ResBundle rb) {
        // CALL doToolbar
        doToolbar();

        // CALL doIcons
        doIcons();

        // INIT menu item used in Windows menu to select this window.
        if (rb.getSuffix().equals("")) {
            menuItem = new JMenuItem("root");
        } else {
            menuItem = new JMenuItem(rb.getSuffix());
        }

        menuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    EditorWindow.this.setSelected(true);
                    RBEdit.getRBEdit().getMainWindow().setSelectedTab(hiddenComponent);
                } catch (PropertyVetoException ex) {}
            }
        });

        // SET bundle to rb
        resBundle = rb;

        // INIT model
        model = new RBEditTableModel();

        rb.addRBListener(model);
        Project proj = null;
	if (RBEdit.getRBEdit() != null) {
            proj = RBEdit.getRBEdit().getProject();
	}

        // INIT table with model and custom Editor and Renderer
        JScrollPane pane = new JScrollPane(table = new JTable(model));
        table.setBounds(25, 5, 300, 220);
        table.getColumnModel().getColumn(1).setCellEditor(editor =
                new RBEditTableCellEditor());
        table.addMouseListener(new PopupListener());

        renderer = new RBEditTableCellRenderer();
        for (int ndx = 0; ndx < 4; ndx++) {
            table.getColumnModel().getColumn(ndx).setCellRenderer(renderer);
        }

        if (resBundle.getSuffix().equals("") || resBundle.isLooseFile()) {
            table.removeColumn(table.getColumn(model.getColumnName(2)));
        }

        // ADD toolbar panel
        getContentPane().add(toolbarPanel, "North");

        // ADD pane containing table
        getContentPane().add(pane, "Center");

        // SET size of editor window
        setBounds(25, 5, 300, 220);
    }

    /**
     Lays out the toolbar

     PRE:  gui elements are not null
     POST: toolbar has been initialized
    */
    public void doToolbar() {
        // INIT toolbar - created by Visual Cafe

        toolbarPanel.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));

        addKeyButton.setDefaultCapable(false);
//      addKeyButton.setToolTipText(bundle.getString("editComButtonToolTip"));
        toolbar.add(addKeyButton);

        remKeyButton.setDefaultCapable(false);
//      remKeyButton.setToolTipText(bundle.getString("editComButtonToolTip"));
        toolbar.add(remKeyButton);

        toolbar.add(new JToolBar.Separator());

        editValButton.setDefaultCapable(false);
//      editValButton.setToolTipText(bundle.getString("editValButtonToolTip"));
        toolbar.add(editValButton);

        editComButton.setDefaultCapable(false);
//      editComButton.setToolTipText(bundle.getString("editComButtonToolTip"));
        toolbar.add(editComButton);

        toolbar.setFloatable(false);
        toolbar.setAlignmentY(0.222222F);
        toolbarPanel.add(toolbar);
    }

    /**
     Adds icons to GUI components

     PRE:  gui elements are not null
     POST: Icons have been added to gui components
    */
    public void doIcons() {
        // INIT icons - created by Visual Cafe
        addKeyIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("addKeyIcon")));
        remKeyIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("remKeyIcon")));
        editValIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("editValIcon")));
        editComIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("editComIcon")));

        addKeyButton.setIcon(addKeyIcon);
        remKeyButton.setIcon(remKeyIcon);
        editValButton.setIcon(editValIcon);
        editComButton.setIcon(editComIcon);
    }

    /**
     Adds a popup menu to the EditorWindow

     PRE: table exists
     POST: table has a popup menu
    */
    public void doPopupMenu() {
        // ADD and INIT popup menu items
        popup.add("Add Key").addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                RBEdit.getRBEdit().getMainWindow().doAddKey();
            }
        });

        popup.add("Rename Key").addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                new RenameKeyDlg();
            }
        });

        popup.add("Remove Key").addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                RBEdit.getRBEdit().getMainWindow().doRemoveKey();
            }
        });

        popup.addSeparator();

        popup.add("Edit Value").addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                table.editCellAt(table.getSelectedRow(), valCol);
            }
        });

        popup.add("Edit Comments").addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                table.editCellAt(table.getSelectedRow(), comCol);
            }
        });

        table.add(popup);
    }

    /**
     Adds necessary listeners

     PRE:  gui components have been initialized
     POST: gui componenets have listeners
    */
    public void doAddListeners() {
        // ADD internal frame listener to various GUI components
        this.addInternalFrameListener(new InternalFrameAdapter() {
            public void internalFrameClosing(InternalFrameEvent e) {
                if (!resBundle.isLooseFile() || !resBundle.isDirty()) {
                    dispose();
                    RBEdit.getRBEdit().getMainWindow().removeWindow(
                        EditorWindow.this);
                    return;
                }

                int result;

                result = JOptionPane.showConfirmDialog(EditorWindow.this,
                        bundle.getString("closeLooseWarningMessage") + " " +
                        resBundle.getFile().getPath() + "?",
                        bundle.getString("closeLooseFileDlg"),
                        JOptionPane.YES_NO_CANCEL_OPTION);

                if (result == JOptionPane.CANCEL_OPTION) {
                    return;
                }

                if (result == JOptionPane.YES_OPTION) {
                    RBEdit.getRBEdit().getProject().saveBundle(resBundle);
                }

                RBEdit.getRBEdit().getMainWindow().removeWindow(EditorWindow.this);
                dispose();
            }

            public void internalFrameActivated(InternalFrameEvent e) {
                RBEdit.getRBEdit().getMainWindow().setSelectedTab(hiddenComponent);
            }
        });

        // ADD action listener to various GUI components
        addKeyButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                RBEdit.getRBEdit().getMainWindow().doAddKey();
            }
        });

        remKeyButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                RBEdit.getRBEdit().getMainWindow().doRemoveKey();
            }
        });

        editValButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                table.editCellAt(table.getSelectedRow(), valCol);
            }
        });

        editComButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                table.editCellAt(table.getSelectedRow(), comCol);
            }
        });
    }

    public void editCurVal() {
        table.editCellAt(table.getSelectedRow(), valCol);
    }

    public void editCurCom() {
        table.editCellAt(table.getSelectedRow(), comCol);
    }

    public JMenuItem getMenuItem() {
        return menuItem;
    }

    public String getTabName() {
        if (resBundle.getSuffix().equals("")) {
            return "root";
        } else {
            return resBundle.getSuffix();
        }
    }

    public String getTooltipText() {
        return "Editor Window";
    }

    /**
     Returns whether this editor window has been modified
     @returns whether this editor window has been modified

     PRE: none
     POST: none
    */
    public boolean isDirty() {
        return resBundle.isDirty();
    }

    /**
     Returns whether this EditorWindow represents a loose file.

     PRE: none
     POST: none
    */
    public boolean isLooseFile() {
        return resBundle.isLooseFile();
    }

    /**
     Returns the ResBundle which this EditorWindow draws
     @return the ResBundle
    */
    public ResBundle getBundle() {
        return resBundle;
    }

    public Dimension getPreferredSize() {
        return new Dimension (320,240);
    }

    /**
     Gets the entry that is currently selected in the Editor Window
     @returns an Entry to be worked with

     PRE: This editor window is open.
     POST: The sel/ selected then null is returned.
    */
    public Entry getSelectedEntry() {
        if (table.getSelectedRow() < 0) {
            return null;
        }

        // RETURN selected value
        return resBundle.getEntry(table.getSelectedRow());
    }
}
