/* Generated by Together */

package net.quartzware;

import java.lang.*;
import java.util.*;
import javax.swing.*;

/**
 Entry
 Encapsulates the data and function of an entry in the JTable in an EditorWindow
 This is equivalent to one row in an EditorWindow.
 @author Chris Perluss
 @version 0.7.1
*/

// Version History
//     11/30/00 - Version 0.9   Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//     11/29/00 - Version 0.7.1 - Anthony Tomarchio
//                source code checked over
//     11-16-00 - Version 0.3 Tony Johansen
//                added PRE and POST conditions
//     11-15-00 - Version 0.2 Susan Hunton
//                added algorithm pseudocode
//     10-30-00 - version 0.1 Chris Perluss
//                created Java Doc

public class Entry implements Cloneable {
    /**
     Constructs an Entry from the components of an entry
     @param key the unique key used to identify this entry
     @param val the value that this entry will hold
     @param comment a comment about the value

     PRE: owner, key, val, and comment are not null
     POST: This class is created
    */
    public Entry(ResBundle owner, String key, Value val, String comment,
            boolean unique) {
        // SET owner
        this.owner = owner;

        // SET key
        this.key = key;

        // SET value
        this.value = val;

        // SET commment
        this.comment = comment;

        this.unique = unique;
    }

    /**
     This function will retrieve the proper item at index ndx
     @return the item at index ndx

     PRE: 0 <= ndx <= 3
     POST: The value is returned
    */
    public Object elementAt(ResBundle ref, int ndx) {
        //+ ASSERT(ndx >= 0 && ndx < 4);

        if (ndx == 0) {
            return key;
        } else if (ndx == 1) {
            return value;
        } else if (ndx == 2) {
            //+ ASSERT(owner != null);
            //+ ASSERT(parent != null);

            Entry ent = owner.getParentEntry(key);
            if (ent != null) {
                return ent.getValue();
            } else {
                return "Parent not found";
            }
        }
        else if (ndx == 3) {
            return comment;
        } else {
            return "";
        }
    }

    /**
     This function will allow the user to edit the value in this entry.  It
     returns true if in-label editing is required, but false if a dialog will
     handle the editing.
     @return Whether to do in label editing of this value

     PRE: This entry is selected in the editor window.
     POST: The value that this entry contains may be modified.
    */
    public boolean editValue() {
        // CALL value.editValue
        return value.editValue();
    }

    /**
     Returns the unique key name in this entry.
     @return the key owned by this Entry
    */
    public String getKey() {
        // RETURN key
        return key;
    }

    /**
     Sets the unique key name in this entry.
     @param the key owned by this Entry
    */
    public void setKey(String keyName) {
        // SET key
        key = keyName;
    }

    /**
     Returns the value of this Entry.
     @return the value of this Entry
    */
    public Value getValue() {
        // RETURN value
        return value;
    }

    /**
     Sets the value for this Entry
     @param the new val for this Entry
    */
    public void setValue(String val) {
        // SET value
        value.setValue(val);
    }

    /**
     Sets the value for this Entry
     @param the new val for this Entry
    */
    public void setValue(Value val) {
        // SET value
        try {
            value = (Value)val.clone();
        } catch (CloneNotSupportedException ex) {}
    }

    /**
     Returns the comment for this Entry
     @return the comment for this Entry
    */
    public String getComment() {
        // RETURN comment
        return comment;
    }

    /**
     Sets the comment for this Entry
     @param com the new comment.
    */
    public void setComment(String com) {
        comment = com;
    }

    /**
     Returns whether this entry is inherited from the parent
     @return false is we are inheriting the parent's value
    */
    public boolean isUnique() {
        // RETURN unique
        return unique;
    }

    /**
     Sets whether this entry is inherited from the parent
     @param uniq true if this entry is inherited from the parent
    */
    public void setUnique(boolean uniq) {
        unique = uniq;
    }

    /**
     Returns a reference to the ResBundle which owns this Entry.
     @return a ResBundle
    */
    public ResBundle getOwner() {
        // RETURN owner
        return owner;
    }

    /**
     Sets a reference to the ResBundle which owns this Entry.
     @param a ResBundle
    */
    public void setOwner(ResBundle rb) {
        // SET owner
        owner = rb;
    }

    /**
     Returns the value of the parent of this bundle
     @return the value of the parent
    */
    public Value getParentValue() {
        // GET and RETURN owner's value
        return owner.getParentEntry(key).getValue();
    }

    protected Object clone() throws CloneNotSupportedException {
        return new Entry(owner, new String(key), (Value)value.clone(),
                new String(comment), unique);
    }

    private String comment;
    private String key;
    private boolean unique;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     */
    private Value value;

    /**
     * @clientCardinality 0..*
     * @supplierCardinality 1
     */
    private ResBundle owner;
}
