package net.quartzware;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import java.util.*;
import java.net.*;
import java.io.File;


/**
 * JAboutDialog
 * A basic implementation of the JDialog class.
 * Shows application information
 */

// Version History
//     11/30/00 - Version 0.9   Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//     11/29/00 - Version 0.7.1 Susan Hunton
//                modified adt header
//              - Anthony Tomarchio - Internationalized

public class JAboutDialog extends CenterDlg {
    JButton okButton = new JButton();
    JLabel aboutLabel = new JLabel();
    ImgPanel aboutDialogPanel = new ImgPanel();

    //Internationalization variables
    Locale currentLocale = Locale.getDefault();
    ResourceBundle bundle = ResourceBundle.getBundle("RBEdit", currentLocale);

    public JAboutDialog(Frame parentFrame) {
        super(parentFrame);

        // CALL doTheLayout
        doTheLayout();

        // CALL addListeners
        addListeners();

        // DISPLAY this dialog
        pack();

        // CALL setVisible
        setVisible(true);
    }

    /**
     This function handles the layout of the Delete Key dialog box

     PRE: attributes have been set
     POST: GUI components have been added to the dlg and arranged appropriately.
    */
    private void doTheLayout() {
        setTitle(bundle.getString("jAboutDialog"));
        setResizable(false);
        setModal(true);
        getContentPane().setLayout(null);
        setSize(400,125);
        setVisible(false);
        okButton.setText(bundle.getString("okButton"));
        okButton.setOpaque(false);
        okButton.setMnemonic((int)'O');
        getContentPane().add(okButton);
        okButton.setBounds(112,71,51,25);
        aboutLabel.setHorizontalAlignment(SwingConstants.CENTER);
        aboutLabel.setText(bundle.getString("aboutLabel"));
        aboutLabel.setDoubleBuffered(true);
        getContentPane().add(aboutLabel);
        aboutLabel.setForeground(java.awt.Color.black);
        aboutLabel.setBounds(95,24,300,25);
        aboutDialogPanel.add(new ImageIcon(RBEdit.class.getResource(
            bundle.getString("aboutDialogPanel"))).getImage());

        aboutDialogPanel.setLayout(null);
        aboutDialogPanel.setBounds(12,12,76,51);
        getContentPane().add(aboutDialogPanel);
    }

    /**
     This function adds actionListeners to buttons which need to be watched.

     PRE: all GUI components have been initialized.
     POST: Listeners added to all componenets that need one.
    */
    private void addListeners() {
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });
    }
    /**
     Shows or hides dialog based on 'b'.
    */
    public void setVisible(boolean vis)
    {
        if (vis) {
            Rectangle bounds = (getParent()).getBounds();
            Dimension size = getSize();
            setLocation(bounds.x + (bounds.width - size.width)/2,
                        bounds.y + (bounds.height - size.height)/2);
        }

        super.setVisible(vis);
    }
}