/* Generated by Together */

package net.quartzware;

import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import javax.swing.*;

/**
 JavaIO is a class which is able to both read to and from .java resource
 bundle files.  It uses the ListResourceBundle class.
 @author Chris Perluss
 @version 0.1
*/

// Version History
//   2/06/01 - Version 0.1 Chris Perluss
//             Creation

public class JavaIO implements RBReader, RBWriter {

    /**
     Reads the File pointed to by file and returns a ResBundle
     @param file Where to read this bundle from
     @return a ResBundle created from the file pointed to by file
    */
    public ResBundle readBundle(File file) {
        ResBundle bundle = new ResBundle("", file, false);  // bundle to return
        BufferedReader in;                   // Allows you to read from a file
        Vector entries;                      // entries which will be read in

        String key;                          // current KeyName
        Value val;                           // current Value
        String lastCom = new String("");     // last comment found

        String line;
        boolean isListRB = false;      // is the class in the .java file derived
                                       // from ListResourceBundle?

        try {
            in = new BufferedReader(new FileReader(file.getPath()));
            line = in.readLine().trim();  // Initialize loop control variable

            // Walk past header information.  Real information starts when {"
            // are the first two character on a line (ignoring whitespace).
            // also check to make sure this is a ListResourceBundle
            while (in.ready() && !line.startsWith("{\"") &&
                    !line.startsWith("{ \"")) {
                if (line.lastIndexOf("extends ListResourceBundle") >= 0) {
                    isListRB = true;
                }
                line = in.readLine();
                if (line == null) {
                    return bundle;
				}
                line = line.trim();
            }

            if (!isListRB) {
                throw new Exception("Not a ListResourceBundle");
            }

            String endLine = System.getProperty("line.separator");
            while (in.ready()) {

                if (line.equals("") || line.equals(endLine)) {               // blank line.  skip it.
                    // do nothing
                } else if (line.startsWith("/")) {   // comment

                    // walk through all of the characters in the string.
                    for (int ndx = 0; ndx < line.length(); ndx++) {
                        char chr = line.charAt(ndx);

                        if (Character.isLetterOrDigit(chr)) { //Has alphanumeric
                            lastCom = line.substring(ndx);    //must be a real
                            break;                      //comment (not a banner)
                        }
                    }
                } else if (line.startsWith("{\"") || line.startsWith("{ \"")) {
                    int quote1 = line.indexOf('"'); // index of first " plus 1
                    key = line.substring(quote1+1, line.indexOf('"', quote1+1));
                    val = processValue(line.substring(line.indexOf(',')+1,
                                line.indexOf('}')).trim());
                    bundle.addEntry(new Entry(bundle, key, val, lastCom, true));
                    lastCom = new String("");
                } else {
                    break;
                }

                line = in.readLine().trim();
            }
        } catch (FileNotFoundException ex) {
            JOptionPane.showMessageDialog(null, "Could not find file "
                    + file.getName(), "Error", JOptionPane.ERROR_MESSAGE);
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(null, ex.getMessage(),
                    "Error", JOptionPane.ERROR_MESSAGE);
            return null;
        }
        return bundle;
    }

    /**
     Writes the resource bundle out to the file
     @param rb the bundle to write out.
     @param file where to write out the bundle to.
    */
    public void writeBundle(ResBundle rb, File file) {
        try {
            BufferedWriter out = new BufferedWriter(new
                    FileWriter(file.getPath()));
            String pad = "        ";  // 8 spaces for padding

            writeln(out, "import java.util.*;");
            out.newLine();

            writeln(out, "public class " + file.getName().substring(0,
                    file.getName().indexOf('.')) +
                    " extends ListResourceBundle {");
            out.newLine();
            writeln(out, "    final Object[][] contents = {");

            Vector entries = rb.getAllEntries();
            for (int ndx = 0; ndx < entries.size(); ndx++) {
                Entry ent = (Entry)entries.elementAt(ndx);

                if (!ent.isUnique()) {
					continue;
				}

                String com = ent.getComment();

                if (!com.trim().equals("")) {
                    out.newLine();
                    writeln(out, pad + "// " + com);
                }

                writeln(out, pad + "{\"" + ent.getKey() + "\", \"" +
                        ent.getValue() + "\"}" +
                        (ndx == entries.size()-1 ? "" : ","));
            }

            writeln(out, "    };");
            out.newLine();
            writeln(out, "    public Object[][] getContents() {return contents;}");
            writeln(out, "}");

            out.flush();
            out.close();
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(null, "Error saving to file "
                    + file.getName(), "Error", JOptionPane.ERROR_MESSAGE);
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(null, ex.getMessage(),
                    "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    protected void writeln(BufferedWriter out, String text) throws Exception {
        out.write(text, 0, text.length());
        out.newLine();
    }

    protected Value processValue(String val) throws Exception {
        //+ ASSERT((val.startsWith("\"") && val.endsWith("\"")) ||
        //+        (!val.startsWith("\"") && !val.endsWith("\""));
        if (val.startsWith("\"")) {    // Could be String. Message or BinaryPath
            if (val.indexOf("{0") >= 0) {
                return new Message(val.substring(1, val.length()-1));
            } else {
                return new StringVal(val.substring(1, val.length()-1));
            }
        } else if (val.startsWith("new ")) {   // could be IntArray, StringArray
                                              // or ObjectVal
            String type = val.substring(3).trim(); // the type, such as "int[]"
            if (type.startsWith("int[]")) {
                StringTokenizer st = new StringTokenizer(type.substring(
                        type.indexOf('{')+1, type.indexOf('}')), ",");

                int[] array = new int[st.countTokens()];
                for (int ndx = 0; st.hasMoreTokens(); ndx++) {
                    array[ndx] = Integer.parseInt(st.nextToken());
                }
                return new IntArray(array);
            } else if (type.startsWith("String[]")) {
                StringTokenizer st = new StringTokenizer(type.substring(
                        type.indexOf('{')+1, type.indexOf('}')), ",");

                String[] array = new String[st.countTokens()];
                for (int ndx = 0; st.hasMoreTokens(); ndx++) {
                    array[ndx] = st.nextToken();
                }
                return new StringArray(array);
            } else if (type.startsWith("Integer")) {
                return new IntegerVal(Integer.valueOf(type.substring(
                        type.indexOf('(')+1, type.indexOf(')'))));
            } else {
                return new ObjectVal(val);
            }
        } else {
            throw new Exception("Unable to parse text");
        }
    }
}
