package net.quartzware;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import java.net.*;
import java.io.File;
import java.util.*;

/**
 KeyDoesNotExistDlg
 Dialog box to alert user that a key is non-existant
 in the Root.
 @author Chris Perluss
 @version 0.7.1
*/

// Version History
//     11/30/00 - Version 0.9   Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//     11/29/00 - Version 0.7.1 - Anthony Tomarchio
//                   Internationalized
//     11/19/00 - Version 0.5 Albert Revisions
//     11/17/00 - Version 0.3 Tony J and Chris
//                wrote PRE and POST conditions
//     11/15/00 - Version 0.2 Chris Perluss
//                wrote algorithm pseudocode
//     10/30/00 - Version 0.1 Chris Perluss
//                wrote ADT headers
public class KeyDoesNotExistDlg extends CenterDlg {
    ImgPanel keyNotExistPanel = new ImgPanel();
    JLabel keyNotExistKeyLabel = new JLabel();
    JLabel keyNotExistValueLabel = new JLabel();
    JLabel keyNotExistCommentLabel = new JLabel();
    JLabel keyNotExistKeyValue = new JLabel();
    JLabel keyNotExistValueValue = new JLabel();
    JLabel keyNotExistCommentValue = new JLabel();
    JPanel rbPanel = new JPanel();
    ButtonGroup rbGroup = new ButtonGroup();
    JRadioButton addKeyRB = new JRadioButton();
    JRadioButton deleteKeyRB = new JRadioButton();
    JButton okButton = new JButton();
    JButton cancelButton = new JButton();
    JTextArea noKeyMsg = new JTextArea();

    //Internationalization variables
    Locale currentLocale = Locale.getDefault();
    ResourceBundle bundle = ResourceBundle.getBundle("RBEdit", currentLocale);

    /**
     Constructs the KeyDoesNotExistDlg
     PRE:  parent cannot be null
     POST:  dialog is created
    */
    public KeyDoesNotExistDlg(Frame parent) {
        super(parent);

        // CALL doTheLayout
        doTheLayout();

        // ADD Listeners
        addListeners();

    // DISPLAY the dialog
    pack();
        setVisible(true);
    }

    /**
     Constructs the KeyDoesNotExistDlg
     PRE:  none
     POST:  dialog is created
    */
    public KeyDoesNotExistDlg() {
        this((Frame)null);
    }

    /**
     Constructs the KeyDoesNotExistDlg and sets title to sTitle
     PRE:  sTitle cannot be null
     POST:  dialog is created
    */
    public KeyDoesNotExistDlg(String sTitle)    {
        this();
        setTitle(sTitle);
    }

    /**
     This function handles the layout of the Find dialog box

     PRE: attributes have been set
     POST: GUI components have been added to the dlg and arranged appropriately.
    */
    private void doTheLayout() {
        setTitle(bundle.getString("keyDoesNotExistDlg"));
        setResizable(false);
        setModal(true);
        getContentPane().setLayout(null);
        setSize(361,233);
        setVisible(false);

        keyNotExistPanel.add(new ImageIcon(RBEdit.class.getResource(
            bundle.getString("keyNotExistPanel"))).getImage());
        keyNotExistPanel.setLayout(null);
        keyNotExistPanel.setBounds(12,12,80,65);
        getContentPane().add(keyNotExistPanel);

        keyNotExistKeyLabel.setText(bundle.getString("keyNotExistKeyLabel"));
        keyNotExistKeyLabel.setForeground(java.awt.Color.black);
        keyNotExistKeyLabel.setBounds(12,84,75,25);
        getContentPane().add(keyNotExistKeyLabel);

        keyNotExistValueLabel.setText(bundle.getString("keyNotExistValueLabel"));
        keyNotExistValueLabel.setForeground(java.awt.Color.black);
        keyNotExistValueLabel.setBounds(12,120,75,25);
        getContentPane().add(keyNotExistValueLabel);

        keyNotExistCommentLabel.setText(bundle.getString("keyNotExistCommentLabel"));
        keyNotExistCommentLabel.setForeground(java.awt.Color.black);
        keyNotExistCommentLabel.setBounds(12,156,75,25);
        getContentPane().add(keyNotExistCommentLabel);

        keyNotExistKeyValue.setText(bundle.getString("keyNotExistKeyValue"));
        keyNotExistKeyValue.setForeground(java.awt.Color.black);
        keyNotExistKeyValue.setFont(new Font("Dialog", Font.BOLD, 12));
        keyNotExistKeyValue.setBounds(84,84,75,25);
        getContentPane().add(keyNotExistKeyValue);

        keyNotExistValueValue.setText(bundle.getString("keyNotExistValueValue"));
        keyNotExistValueValue.setForeground(java.awt.Color.black);
        keyNotExistValueValue.setBounds(84,120,75,25);
        getContentPane().add(keyNotExistValueValue);

        keyNotExistCommentValue.setText(bundle.getString("keyNotExistCommentValue"));
        keyNotExistCommentValue.setForeground(java.awt.Color.black);
        keyNotExistCommentValue.setBounds(84,156,100,25);
        getContentPane().add(keyNotExistCommentValue);

        addKeyRB.setText(bundle.getString("addKeyRB"));
        addKeyRB.setBounds(0,0,150,25);
        rbGroup.add(addKeyRB);
        rbPanel.add(addKeyRB);

        deleteKeyRB.setText(bundle.getString("deleteKeyRB"));
        deleteKeyRB.setBounds(0,36,150,25);
        rbGroup.add(deleteKeyRB);
        rbPanel.add(deleteKeyRB);

        rbPanel.setLayout(null);
        rbPanel.setBounds(168,84,175,75);
        getContentPane().add(rbPanel);

        okButton.setText(bundle.getString("okButton"));
        okButton.setActionCommand("OK");
        okButton.setBounds(132,192,100,25);
        getContentPane().add(okButton);

        cancelButton.setText(bundle.getString("cancelButton"));
        cancelButton.setActionCommand("Cancel");
        cancelButton.setBounds(240,192,100,25);
        getContentPane().add(cancelButton);

        noKeyMsg.setText(bundle.getString("noKeyMsg"));
        noKeyMsg.setOpaque(false);
        noKeyMsg.setEditable(false);
        noKeyMsg.setLineWrap(true);

        getContentPane().add(noKeyMsg);
        noKeyMsg.setFont(new Font("Dialog", Font.BOLD, 12));
        noKeyMsg.setBounds(96,24,252,40);
    }

    /**
     This function adds actionListeners to buttons which need to be watched.

     PRE: all GUI components have been initialized.
     POST: Listeners added to all componenets that need one.
    */
    private void addListeners() {
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });

        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });
    }

    public void setVisible(boolean b) {
        if (b) {
            setLocation(50, 50);
        }
        super.setVisible(b);
    }
}
