/* Generated by Together */

package net.quartzware;

import javax.swing.*;
import java.lang.*;
import java.util.*;
import java.io.File;
import java.awt.event.*;
import javax.swing.event.*;
import java.beans.*;
import java.awt.*;
import java.net.*;
import javax.swing.border.*;

/**
 MainWindow
 Main window for the RBEdit application.  Handles GUI from the menu system.
 @author Chris Perluss
 @version 0.7.1
*/

// Version History
//     03/05/02 - Tony T - Implemented Help System
//     02/25/01 - Version 1.0   Tony J. wrote doRemoveKey function.
//     02/04/01 - Version 0.95  Al implemented Save As Function
//     11/30/00 - Version 0.9   Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//              - Anthony Tomarchio - Added various icons
//              - Anthony Tomarchio - AddKey, RenameKey and DeleteKey edit menu
//                items now bring up dialog boxes.
//     11/29/00 - Version 0.7.1 - Susan Hunton
//                modified adt header, added pseudocode
//     11/28/00 - Version 0.7 - Chris Perluss
//                Added AddKey and RemoveKey to the toolbar.  Added AddKey,
//                RenameKey, and RemoveKey to the Edit Menu.
//     11/20/00 - Version 0.6 - Anthony Tomarchio
//                Updated for internationalization
//     11/18/00 - Version 0.5 Chris Perluss
//                inserted GUI code
//     11/17/00 - Version 0.3 Tony J and Chris Perluss
//                wrote PRE and POST conditions
//     11/15/00 - Version 0.2 Chris Perluss
//                wrote algorithm pseudocode
//     10/30/00 - Version 0.1 Chris Perluss
//                wrote ADT headers

// This class creates the main window and its' icons
public class MainWindow extends JFrame {
    ImageIcon newIcon;
    ImageIcon openIcon;
    ImageIcon saveIcon;
    ImageIcon cutIcon;
    ImageIcon copyIcon;
    ImageIcon pasteIcon;
    ImageIcon addKeyIcon;
    ImageIcon remKeyIcon;
    ImageIcon aboutIcon;
    ImageIcon undoIcon;
    ImageIcon redoIcon;

    JDesktopPane desktop = new JDesktopPane();

    JLayeredPane editorPanel;
    JPanel bottomPanel;

    JTabbedPane ezPane;

    FileDialog saveDlg = new FileDialog(this);
    FileDialog openDlg = new FileDialog(this);
    JPanel toolbarPanel = new JPanel();
    JToolBar toolbar = new JToolBar();
    JButton newProjectButton = new JButton();
    JButton openButton = new JButton();
    JButton saveButton = new JButton();
    JButton cutButton = new JButton();
    JButton copyButton = new JButton();
    JButton pasteButton = new JButton();
    JButton addKeyButton = new JButton();
    JButton remKeyButton = new JButton();
    JButton aboutButton = new JButton();

    JMenuBar mainMenuBar = new JMenuBar();

    JMenu fileMenu = new JMenu();
    JMenuItem miNewProject = new JMenuItem();
    JMenuItem miOpen = new JMenuItem();
    JMenuItem miOpenProject = new JMenuItem();
    JMenuItem miClose = new JMenuItem();
    JMenuItem miSave = new JMenuItem();
    JMenuItem miSaveProjectAs = new JMenuItem();
    JMenuItem miExit = new JMenuItem();

    JMenu editMenu = new JMenu();
    JMenuItem miUndo = new JMenuItem();
    JMenuItem miRedo = new JMenuItem();
    JMenuItem miCut = new JMenuItem();
    JMenuItem miCopy = new JMenuItem();
    JMenuItem miPaste = new JMenuItem();
    JMenuItem miFind = new JMenuItem();

    JMenu projectMenu = new JMenu();
    JMenuItem miNewBundle = new JMenuItem();
    JMenuItem miImportBundle = new JMenuItem();
    JMenuItem miRemoveBundle = new JMenuItem();
    JMenuItem miPreferences = new JMenuItem();

    JMenu bundleMenu = new JMenu();
    JMenuItem miAddKey = new JMenuItem();
    JMenuItem miRenameKey = new JMenuItem();
    JMenuItem miDeleteKey = new JMenuItem();
    JMenuItem miEditValue = new JMenuItem();
    JMenuItem miEditComments = new JMenuItem();

    JMenu viewMenu = new JMenu();
    JCheckBoxMenuItem miShowKeys = new JCheckBoxMenuItem("",true);
    JCheckBoxMenuItem miShowValues = new JCheckBoxMenuItem("",true);
    JCheckBoxMenuItem miShowParentsValue = new JCheckBoxMenuItem("",true);
    JCheckBoxMenuItem miShowComments = new JCheckBoxMenuItem("",true);

    JMenu windowMenu = new JMenu();
    JMenuItem miCascadeWindows = new JMenuItem();
    JMenuItem miTileHorizontally = new JMenuItem();
    JMenuItem miTileVertically = new JMenuItem();

    JMenu helpMenu = new JMenu();
    JMenuItem miHelpTopics = new JMenuItem();
    JMenuItem miAbout = new JMenuItem();

    /**
     * @clientCardinality 1
     * @supplierCardinality 0..*
     */
    Vector windows = new Vector();
    JFileChooser fileChooser;
    RBEdit rbedit;
    boolean dirty = true;
    HelpDlg helpSystem = new HelpDlg();

    //Internationalization variables
    Locale currentLocale = Locale.getDefault();
    ResourceBundle bundle = ResourceBundle.getBundle("RBEdit", currentLocale);

    /**
     Default Constructor
     @param rbedit RBEdit that this window draws.

     PRE: rbedit is not null
     POST: the class is constructed
    */
    public MainWindow(RBEdit rbedit) {
        // SET this.rbedit to rbedit
        this.rbedit = rbedit;

        // CALL doTheLayout
        doTheLayout();

        // ADD Listeners
        doAddListeners();

        editorPanel.setLayout(null);

        // DISPLAY the window
        pack();
        setVisible(true);
    }

    /**
     Lays out this window

     PRE: this class's attributes have been set
     POST: GUI components have been added to the window and arranged appropriately.
    */
    public void doTheLayout() {
        desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
        setContentPane(desktop);

        getContentPane().setLayout(new BorderLayout());

        setTitle(bundle.getString("mainWindow"));
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        setSize(640,480);

        saveDlg.setMode(FileDialog.SAVE);
        saveDlg.setTitle(bundle.getString("saveDlg"));
        openDlg.setMode(FileDialog.LOAD);
        openDlg.setTitle(bundle.getString("openDlg"));

        doIcons();

        doFileMenu();
        doEditMenu();
        doProjectMenu();
        doBundleMenu();
        doViewMenu();
        doWindowsMenu();
        doHelpMenu();
        setJMenuBar(mainMenuBar);

        doToolbar();

        bottomPanel = new JPanel(new BorderLayout());
        doEZTabs();
        doStatusBar();

        editorPanel = new JLayeredPane();
        editorPanel.setOpaque(false);

//        getContentPane().add(toolbarPanel, BorderLayout.NORTH);
        getContentPane().add(editorPanel);

        /* Temporary hack to get the EditorWindows to draw correctly */
        JPanel btm2Panel = new JPanel();
        btm2Panel.setLayout(new BorderLayout());
        btm2Panel.add(toolbarPanel, BorderLayout.NORTH);
        btm2Panel.add(bottomPanel, BorderLayout.SOUTH);
        /* End of hack */

        getContentPane().add(btm2Panel, BorderLayout.SOUTH);
    }

    /**
     Lays out the File Menu, sets the file menu items to the correct langauge

     PRE: bundle points to the correct Resource Bundle for the current language
     POST: File Menu has been initialized
    */
    public void doFileMenu() {
        fileMenu.setText(bundle.getString("fileMenu"));
        fileMenu.setMnemonic((int)'F');
        mainMenuBar.add(fileMenu);

        miNewProject.setText(bundle.getString("miNewProject"));
        miNewProject.setMnemonic((int)'N');
        fileMenu.add(miNewProject);

        miOpen.setText(bundle.getString("miOpen"));
        miOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.CTRL_MASK));
        miOpen.setMnemonic((int)'O');
        fileMenu.add(miOpen);

        miOpenProject.setText(bundle.getString("miOpenProject"));
        miOpenProject.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.SHIFT_MASK|Event.CTRL_MASK));
        miOpenProject.setMnemonic((int)'P');
        fileMenu.add(miOpenProject);

        miClose.setText(bundle.getString("miClose"));
        miClose.setMnemonic((int)'C');
        fileMenu.add(miClose);

        fileMenu.add(new JSeparator());

        miSave.setText(bundle.getString("miSave"));
        miSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK));
        miSave.setMnemonic((int)'S');
        fileMenu.add(miSave);

        miSaveProjectAs.setText(bundle.getString("miSaveProjectAs"));
        miSaveProjectAs.setMnemonic((int)'A');
        fileMenu.add(miSaveProjectAs);

        fileMenu.add(new JSeparator());

        miExit.setText(bundle.getString("miExit"));
        miExit.setMnemonic((int)'X');
        fileMenu.add(miExit);
    }

    /**
     Lays out the Edit Menu, sets the edit menu items to the correct langauge

     PRE:  bundle points to the correct Resource Bundle for the current language
     POST: Edit Menu has been initialized
    */
    public void doEditMenu() {
        editMenu.setText(bundle.getString("editMenu"));
        editMenu.setDoubleBuffered(true);
        editMenu.setMnemonic((int)'E');
        mainMenuBar.add(editMenu);

        miUndo.setText(bundle.getString("miUndo"));
        miUndo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, Event.CTRL_MASK));
        miUndo.setMnemonic((int)'U');
        editMenu.add(miUndo);

        miRedo.setText(bundle.getString("miRedo"));
        miRedo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, Event.CTRL_MASK));
        miRedo.setMnemonic((int)'R');
        editMenu.add(miRedo);

        editMenu.add(new JSeparator());

        miCut.setText(bundle.getString("miCut"));
        miCut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, Event.CTRL_MASK));
        miCut.setMnemonic((int)'T');
        editMenu.add(miCut);

        miCopy.setText(bundle.getString("miCopy"));
        miCopy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, Event.CTRL_MASK));
        miCopy.setMnemonic((int)'C');
        editMenu.add(miCopy);

        miPaste.setText(bundle.getString("miPaste"));
        miPaste.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, Event.CTRL_MASK));
        miPaste.setMnemonic((int)'P');
        editMenu.add(miPaste);

        editMenu.add(new JSeparator());

        miFind.setText(bundle.getString("miFind"));
        miFind.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, Event.CTRL_MASK));
        editMenu.add(miFind);
    }

    /**
     Lays out the Project Menu, sets the project menu items to the correct langauge

     PRE:  bundle points to the correct Resource Bundle for the current language
     POST: Edit Menu has been initialized
    */
    public void doProjectMenu() {
        projectMenu.setText(bundle.getString("projectMenu"));
        projectMenu.setDoubleBuffered(true);
        projectMenu.setMnemonic((int)'P');
        mainMenuBar.add(projectMenu);

        miNewBundle.setText(bundle.getString("miNewBundle"));
        miNewBundle.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, Event.CTRL_MASK));
        miNewBundle.setMnemonic((int)'N');
        projectMenu.add(miNewBundle);

        miImportBundle.setText(bundle.getString("miImportBundle"));
        miImportBundle.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, Event.CTRL_MASK));
        miImportBundle.setMnemonic((int)'A');
        projectMenu.add(miImportBundle);

        miRemoveBundle.setText(bundle.getString("miRemoveBundle"));
        miRemoveBundle.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, Event.CTRL_MASK));
        miRemoveBundle.setMnemonic((int)'R');
        projectMenu.add(miRemoveBundle);

        projectMenu.add(new JSeparator());

        miPreferences.setText(bundle.getString("miPreferences"));
        miPreferences.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, Event.CTRL_MASK));
        miPreferences.setMnemonic((int)'P');
        projectMenu.add(miPreferences);
    }
    /**
     Lays out the Bundle Menu, sets the bundle menu items to the correct langauge

     PRE:  bundle points to the correct Resource Bundle for the current language
     POST: Bundle Menu has been initialized
    */
    public void doBundleMenu() {
        bundleMenu.setText(bundle.getString("bundleMenu"));
        bundleMenu.setDoubleBuffered(true);
        bundleMenu.setMnemonic((int)'B');
        mainMenuBar.add(bundleMenu);

        miAddKey.setText(bundle.getString("miAddKey"));
        bundleMenu.add(miAddKey);

        miRenameKey.setText(bundle.getString("miRenameKey"));
        bundleMenu.add(miRenameKey);

        miDeleteKey.setText(bundle.getString("miDeleteKey"));
        bundleMenu.add(miDeleteKey);

        bundleMenu.add(new JSeparator());

        miEditValue.setText(bundle.getString("miEditValue"));
        miEditValue.setMnemonic((int)'A');
        bundleMenu.add(miEditValue);

        miEditComments.setText(bundle.getString("miEditComments"));
        miEditComments.setMnemonic((int)'M');
        bundleMenu.add(miEditComments);

    }

    /**
     Lays out the View Menu, sets the view menu items to the correct langauge

     PRE:  bundle points to the correct Resource Bundle for the current language
     POST: Edit Menu has been initialized
    */
    public void doViewMenu() {
        viewMenu.setText(bundle.getString("viewMenu"));
        viewMenu.setMnemonic((int)'V');
        mainMenuBar.add(viewMenu);

        miShowKeys.setText(bundle.getString("miShowKeys"));
        //miShowKeys.setSelected(true);
        miShowKeys.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, Event.CTRL_MASK));
        miShowKeys.setMnemonic((int)'K');
        viewMenu.add(miShowKeys);

        miShowValues.setText(bundle.getString("miShowValues"));
        //miShowValues.setSelected(true);
        miShowValues.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, Event.CTRL_MASK));
        miShowValues.setMnemonic((int)'V');
        viewMenu.add(miShowValues);

        miShowParentsValue.setText(bundle.getString("miShowParentsValue"));

        miShowParentsValue.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3, Event.CTRL_MASK));
        miShowParentsValue.setMnemonic((int)'P');
        viewMenu.add(miShowParentsValue);

        miShowComments.setText(bundle.getString("miShowComments"));
        miShowComments.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_4, Event.CTRL_MASK));
        miShowComments.setMnemonic((int)'C');
        viewMenu.add(miShowComments);
    }

    /**
     Lays out the Windows Menu, sets the window menu items to the correct langauge

     PRE:  bundle points to the correct Resource Bundle for the current language
     POST: Edit Menu has been initialized
    */
    public void doWindowsMenu() {
        windowMenu.setText(bundle.getString("windowMenu"));
        windowMenu.setMnemonic((int)'W');
        mainMenuBar.add(windowMenu);

        miCascadeWindows.setText(bundle.getString("miCascadeWindows"));
        miCascadeWindows.setMnemonic((int)'C');
        windowMenu.add(miCascadeWindows);

        miTileHorizontally.setText(bundle.getString("miTileHorizontally"));
        miTileHorizontally.setMnemonic((int)'H');
        windowMenu.add(miTileHorizontally);

        miTileVertically.setText(bundle.getString("miTileVertically"));
        miTileVertically.setMnemonic((int)'V');
        windowMenu.add(miTileVertically);

        windowMenu.add(new JSeparator());
    }

    /**
     Lays out the Help Menu, sets the help menu items to the correct langauge

     PRE:  bundle points to the correct Resource Bundle for the current language
     POST: Edit Menu has been initialized
    */
    public void doHelpMenu() {
        helpMenu.setText(bundle.getString("helpMenu"));
        helpMenu.setMnemonic((int)'H');
        mainMenuBar.add(helpMenu);

        miHelpTopics.setText(bundle.getString("miHelpTopics"));
        helpMenu.add(miHelpTopics);

        helpMenu.add(new JSeparator());

        miAbout.setText(bundle.getString("miAbout"));
        miAbout.setMnemonic((int)'A');
        helpMenu.add(miAbout);
    }

    /**
     Adds icons to GUI components

     PRE:  gui elements are not null
     POST: Icons have been added to gui components
    */
    public void doIcons() {
        newIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("newIcon")));
        openIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("openIcon")));
        saveIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("saveIcon")));
        cutIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("cutIcon")));
        copyIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("copyIcon")));
        undoIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("undoIcon")));
        redoIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("redoIcon")));

        pasteIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("pasteIcon")));
        addKeyIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("addKeyIcon")));
        remKeyIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("remKeyIcon")));
        aboutIcon = new ImageIcon(RBEdit.class.getResource(bundle.getString("aboutIcon")));

        //file menu
        miNewProject.setIcon(newIcon);
        miOpen.setIcon(openIcon);
        miOpenProject.setIcon(openIcon);
        //miClose.setIcon(aboutIcon);
        miSave.setIcon(saveIcon);
        miSaveProjectAs.setIcon(saveIcon);
        //miExit.setIcon(aboutIcon);

        //edit menu
        //miUndo.setIcon(undoIcon);
        //miRedo.setIcon(redoIcon);
        miCut.setIcon(cutIcon);
        miCopy.setIcon(copyIcon);
        miPaste.setIcon(pasteIcon);
        //miEditValue.setIcon(aboutIcon);
        //miEditComments.setIcon(aboutIcon);
        //miFind.setIcon(aboutIcon);

        //project menu
        //miNewBundle.setIcon(aboutIcon);
        //miImportBundle.setIcon(aboutIcon);
        //miRemoveBundle.setIcon(aboutIcon);
        //miPreferences.setIcon(aboutIcon);

        //view menu
        //miShowKeys.setIcon(aboutIcon);
        //miSHowValues.setIcon(aboutIcon);
        //miShowParentsValue.setIcon(aboutIcon);
        //miShowComments.setIcon(aboutIcon);

        //window menu
        //miCascadeWindows.setIcon(aboutIcon);
        //miTileHorizontally.setIcon(aboutIcon);
        //miTileVertically.setIcon(aboutIcon);

        //help menu
        //miHelpTopics.setIcon(aboutIcon);
        miAbout.setIcon(aboutIcon);

        newProjectButton.setIcon(newIcon);
        openButton.setIcon(openIcon);
        saveButton.setIcon(saveIcon);

        cutButton.setIcon(cutIcon);
        copyButton.setIcon(copyIcon);
        pasteButton.setIcon(pasteIcon);

        addKeyButton.setIcon(addKeyIcon);
        remKeyButton.setIcon(remKeyIcon);

        aboutButton.setIcon(aboutIcon);
    }

    /**
     Lays out the EZTabs

     PRE: gui elements are not null
     POST: eztabs has been initialized
    */
    public void doEZTabs() {
        ezPane = new JTabbedPane();
        ezPane.setTabPlacement(SwingConstants.BOTTOM);
        ezPane.setPreferredSize(new Dimension(50, 20));
        bottomPanel.add(ezPane, BorderLayout.NORTH);
    }

    /**
     Lays out the Status Bar

     PRE: gui elements are not null
     POST: statusbar has been initialized
    */
    public void doStatusBar() {
        JPanel statusBar = new JPanel();
        statusBar.setPreferredSize(new Dimension(50, 17));
        statusBar.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        bottomPanel.add(statusBar, BorderLayout.SOUTH);
    }

    /**
     Lays out the toolbar

     PRE:  gui elements are not null
     POST: toolbar has been initialized
    */
    public void doToolbar() {
        toolbarPanel.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));

        newProjectButton.setDefaultCapable(false);
        newProjectButton.setToolTipText(bundle.getString("newProjectButtonToolTip"));
        toolbar.add(newProjectButton);

        openButton.setDefaultCapable(false);
        openButton.setToolTipText(bundle.getString("openButtonToolTip"));
        toolbar.add(openButton);

        saveButton.setDefaultCapable(false);
        saveButton.setToolTipText(bundle.getString("saveButtonToolTip"));
        toolbar.add(saveButton);

        toolbar.add(new JToolBar.Separator());

        cutButton.setDefaultCapable(false);
        cutButton.setToolTipText(bundle.getString("cutButtonToolTip"));
        toolbar.add(cutButton);

        copyButton.setDefaultCapable(false);
        copyButton.setToolTipText(bundle.getString("copyButtonToolTip"));
        toolbar.add(copyButton);

        pasteButton.setDefaultCapable(false);
        pasteButton.setToolTipText(bundle.getString("pasteButtonToolTip"));
        toolbar.add(pasteButton);

        toolbar.add(new JToolBar.Separator());

        addKeyButton.setDefaultCapable(false);
        //addKeyButton.setToolTipText(bundle.getString("pasteButtonToolTip"));
        toolbar.add(addKeyButton);

        remKeyButton.setDefaultCapable(false);
        //remKeyButton.setToolTipText(bundle.getString("pasteButtonToolTip"));
        toolbar.add(remKeyButton);

        toolbar.add(new JToolBar.Separator());

        aboutButton.setDefaultCapable(false);
        aboutButton.setToolTipText(bundle.getString("aboutButtonToolTip"));
        toolbar.add(aboutButton);

        toolbar.setFloatable(false);
        toolbar.setAlignmentY(0.222222F);
        toolbar.setDoubleBuffered(true);
        toolbarPanel.add(toolbar);
    }


    /**
     Adds necessary listeners

     PRE:  gui components have been initialized
     POST: gui componenets have listeners
    */
    public void doAddListeners() {
       // INIT change listener
        ezPane.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                try {
                    Component comp = ezPane.getSelectedComponent();
                    if (comp != null) {
                        ((InternalWindow.HiddenComponent)comp).getFrame().setSelected(true);
                    }
                } catch (PropertyVetoException ex) {}
            }
        });

        // INIT window listener
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        miNewProject.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doNewProject();
            }
        });

        miOpen.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doOpenLooseFile();
            }
        });

        miSave.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doSave();
            }
        });

        miExit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
               if (doClose()) {
                   System.exit(0);
               }
           }
        });

        miAbout.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                new JAboutDialog(MainWindow.this);
            }
        });

        miHelpTopics.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                helpSystem.setVisible(true);
            }
        });

        miFind.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                new FindDlg();
            }
        });

        miClose.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doClose();
            }
        });

        miEditValue.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                if (windows.size() > 1 && desktop.getSelectedFrame() instanceof
                        EditorWindow) {
                    ((EditorWindow)desktop.getSelectedFrame()).editCurVal();
                }
            }
        });

        miEditComments.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                if (windows.size() > 1 && desktop.getSelectedFrame() instanceof
                        EditorWindow) {
                    ((EditorWindow)desktop.getSelectedFrame()).editCurCom();
                }
            }
        });

        miNewBundle.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doNewBundle();
            }
        });

        miImportBundle.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                String title = openDlg.getTitle();
                openDlg.setTitle("Import Bundle");
                openDlg.setVisible(true);
                openDlg.setTitle(title);
            }
        });

        miPreferences.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                if (RBEdit.getRBEdit().getProject() != null) {
                    new PrefDlg(((ProjectWindow)
                            windows.elementAt(0)).getProject().getConfig());
                }
            }
        });

        miRemoveBundle.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                ResBundle rb = ((ProjectWindow)
                        windows.elementAt(0)).getSelectedBundle();

                if (rb.getSuffix().equals("")) {
                    TextArea ta = new TextArea(bundle.getString("remRootError"),
                            6, 25, TextArea.SCROLLBARS_NONE);

                    ta.setEditable(false);
                    JOptionPane.showMessageDialog(MainWindow.this,
                         ta,
                         bundle.getString("error"),
                         JOptionPane.ERROR_MESSAGE);
                    return;
                }

                TextArea ta = new TextArea(bundle.getString("remBundleMsg")
                        + " " + rb.getSuffix() + "?",
                        6, 25, TextArea.SCROLLBARS_NONE);

                ta.setEditable(false);

                int result = JOptionPane.showConfirmDialog(MainWindow.this,
                         ta,
                         bundle.getString("removeBundleDlg"),
                         JOptionPane.OK_CANCEL_OPTION);

                if (result == JOptionPane.CANCEL_OPTION)
                    return;

                // does not process (ndx = 0) because that is the ProjectWindow
                for (int ndx = windows.size()-1; ndx > 0; ndx--) {
                    EditorWindow ew = (EditorWindow)windows.elementAt(ndx);
                    if (ew.getBundle().getSuffix().startsWith(rb.getSuffix())) {
                        ew.dispose();
                        removeWindow(ew);
                    }
                }

                RBEdit.getRBEdit().getProject().removeBundleAndChildren(
                        rb.getSuffix());
            }
        });

        miOpenProject.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doOpenProject();
            }
        });

        miSaveProjectAs.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doSaveAs();
            }
        });

        miAddKey.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doAddKey();
            }
        });

        miRenameKey.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                new RenameKeyDlg();
            }
        });

        miDeleteKey.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doRemoveKey();
            }
        });

        openButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doOpenLooseFile();
            }
        });

        saveButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doSave();
            }
        });

        aboutButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                new JAboutDialog(MainWindow.this);
            }
        });

        newProjectButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doNewProject();
            }
        });

        addKeyButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doAddKey();
            }
        });

        remKeyButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                doRemoveKey();
            }
        });

        fileMenu.addMenuListener(new MenuListener() {
            public void menuCanceled(MenuEvent e) {}
            public void menuDeselected(MenuEvent e) {}

            public void menuSelected(MenuEvent e) {
                boolean projOpen = (RBEdit.getRBEdit().getProject() != null);
                boolean windowSel = (desktop.getSelectedFrame() != null);
                miClose.setEnabled(projOpen || windowSel);
                miSave.setEnabled(windowSel);
                miSaveProjectAs.setEnabled(projOpen);
            }
        });

        editMenu.addMenuListener(new MenuListener() {
            public void menuCanceled(MenuEvent e) {}
            public void menuDeselected(MenuEvent e) {}

            public void menuSelected(MenuEvent e) {
                boolean projOpen = (RBEdit.getRBEdit().getProject() != null);
                boolean windowSel = (desktop.getSelectedFrame() != null);
                miFind.setEnabled(projOpen || windowSel);
            }
        });

        projectMenu.addMenuListener(new MenuListener() {
            public void menuCanceled(MenuEvent e) {}
            public void menuDeselected(MenuEvent e) {}

            public void menuSelected(MenuEvent e) {
                boolean projOpen = (RBEdit.getRBEdit().getProject() != null);
                boolean windowSel = (desktop.getSelectedFrame() != null);

                miNewBundle.setEnabled(projOpen);
                miImportBundle.setEnabled(projOpen);

                miRemoveBundle.setEnabled(false);
                if (projOpen) {
                    ProjectWindow pw = (ProjectWindow)windows.elementAt(0);
                    if (pw.getSelectedBundle() != null &&
                            !pw.getSelectedBundle().getSuffix().equals("")) {
                        miRemoveBundle.setEnabled(true);
                    }
                }
                miPreferences.setEnabled(projOpen);
            }
        });

        bundleMenu.addMenuListener(new MenuListener() {
            public void menuCanceled(MenuEvent e) {}
            public void menuDeselected(MenuEvent e) {}

            public void menuSelected(MenuEvent e) {
                JInternalFrame selFrame = desktop.getSelectedFrame();
                boolean projOpen = (RBEdit.getRBEdit().getProject() != null);
                boolean windowSel = (desktop.getSelectedFrame() != null);
                boolean loose = (windowSel && selFrame instanceof EditorWindow
                        && ((EditorWindow)selFrame).isLooseFile());

                miAddKey.setEnabled(projOpen && !loose);
                miRenameKey.setEnabled(projOpen && !loose);
                miDeleteKey.setEnabled(projOpen && windowSel && !loose);
                miEditValue.setEnabled(windowSel);
                miEditComments.setEnabled(windowSel);
            }
        });

        viewMenu.addMenuListener(new MenuListener() {
            public void menuCanceled(MenuEvent e) {}
            public void menuDeselected(MenuEvent e) {}

            public void menuSelected(MenuEvent e) {
                boolean windowSel = (desktop.getSelectedFrame() != null);

                miShowKeys.setEnabled(windowSel);
                miShowValues.setEnabled(windowSel);
                miShowParentsValue.setEnabled(windowSel);
                JInternalFrame jif = desktop.getSelectedFrame();
                if (jif instanceof EditorWindow &&
                        (((EditorWindow)jif).isLooseFile() || ((EditorWindow)
                        jif).getBundle().getSuffix().equals(""))) {
                    miShowParentsValue.setEnabled(false);
                }

                miShowComments.setEnabled(windowSel);
            }
        });

    }

    /**
     Opens a new Editor Window
     @param bundle The ResBundle to add

     PRE: bundle is not null
     POST: the bundle has an associated EditorWindow
    */
    public void openEditorWindow(ResBundle bundle) {
        for (int ndx = 1; ndx < windows.size(); ndx++)
            if (((EditorWindow)windows.elementAt(ndx)).getBundle() == bundle) {
                return;
            }

        EditorWindow ew = new EditorWindow(bundle);
        addTab(ew, false);
        editorPanel.add(ew);
        ew.setLocation(200, 5);
    }

    public void doAddKey() {
        Project proj = RBEdit.getRBEdit().getProject();
        if (proj == null) {
            return;
        }

        ResBundle rootBundle = proj.getRootBundle();

        AddKeyDlg akDlg = new AddKeyDlg(rootBundle);
        akDlg.setVisible(true);

        if (akDlg.getKeyName() == null || akDlg.getValue() == null) {
            return;
        }

        Entry ent = new Entry(rootBundle, akDlg.getKeyName(),
            akDlg.getValue(), "", true);

        rootBundle.addEntry(ent);

        Vector bundles = proj.getAllBundles();
        for (int ndx = 1; ndx < bundles.size(); ndx++) {
            try {
                Entry newEnt = (Entry)ent.clone();
                ResBundle owner = (ResBundle)bundles.elementAt(ndx);
                newEnt.setOwner(owner);
                newEnt.setUnique(false);
                owner.addEntry(newEnt);
            } catch (CloneNotSupportedException ex) {}
        }
    }

    /**
     Opens up a file dialog so the user can select the file to open.

     PRE: none
     POST: may be a new Editor Window in the main window.
    */
    public void doOpenLooseFile() {
        openDlg.setVisible(true);

        if (openDlg.getFile() == null) {
            return;
        }

        File open = new File(
            openDlg.getDirectory() + openDlg.getFile());

        ResBundle bund = Project.loadBundle(open);
        if (bund == null) {
            return;
        }
        bund.setLooseFile(true);

        //Add new resource bundle to the main window
        openEditorWindow(bund);
    }

    /**
     Pops up a dialog asking the user if they wish to delete
     the selected key

     PRE: none
     POST: a key may be removed from all bundles
    */
    public void doRemoveKey() {
        int result;                //Holds result for warning dialog
        Project proj = RBEdit.getRBEdit().getProject();
        if (proj == null) {
            return;
        }

        //Check if a key is selected
        JInternalFrame selFrame = desktop.getSelectedFrame();
        if (selFrame == null || !(selFrame instanceof EditorWindow)) {
            return;
        }
        Entry ent = ((EditorWindow)selFrame).getSelectedEntry();
        if (ent == null) {
            return;
        }

        TextArea ta = new TextArea(bundle.getString("deleteKeyWarningMessage"),
            6, 25, TextArea.SCROLLBARS_NONE);       //Create the text area for
                                                    //this warning message
        ta.setEditable(false);

        result = JOptionPane.showConfirmDialog(MainWindow.this,
                ta,
                bundle.getString("deleteKeyDlg"),
                JOptionPane.OK_CANCEL_OPTION);
        RBEdit.getRBEdit().getProject().deleteKey(ent.getKey());
    }

    /**
     Pops up a dialog to let the user decide whether to use a new root
     or import an existing one.  If using an existing one, opens up a file
     dialog so the user can select the existing root.

     PRE: none
     POST: may be a new project window in the main window.
    */
    public void doNewBundle() {
        Project proj = RBEdit.getRBEdit().getProject();
        if (proj == null) {
			return;
		}

        ResBundle selBundle =
                ((ProjectWindow)windows.elementAt(0)).getSelectedBundle();

        String curSfx = "";
        String fileType = proj.getConfig().fileType;

        if (selBundle != null) {
            curSfx = selBundle.getSuffix();
        }

        NewBundleDlg dlg = new NewBundleDlg(curSfx, fileType, false);
        dlg.setVisible(true);

        String suffix = dlg.getSuffix();
        String filename = dlg.getFilename();
        fileType = dlg.getFileType();
        if (suffix == null || filename == null) {
            return;
        }

        String test = filename.toLowerCase();

        filename = appendExtension(filename, fileType);

        if (RBEdit.getRBEdit().getProject().getBundle(suffix) != null) {
            JOptionPane.showMessageDialog(null, "Error.  A bundle for "
                + "the language \"" + suffix + "\" alread exists",
                "Error", JOptionPane.ERROR_MESSAGE);
            return;
        }

        String shortSfx = suffix;
        while (shortSfx.lastIndexOf('_') >= 0) {
            shortSfx = shortSfx.substring(0, shortSfx.lastIndexOf('_'));

            if (RBEdit.getRBEdit().getProject().getBundle(shortSfx) == null) {
/*                int result = JOptionPane.showConfirmDialog(MainWindow.this,
                    "Parent Bundle for the language " + shortSfx + " does " +
                    "not exist.  Would you like to create it?",
                    "Parent Bundle Does Not Exist",
                    JOptionPane.OK_CANCEL_OPTION);

                if (result == JOptionPane.CANCEL_OPTION) {
                    return;
                }
*/
                JOptionPane.showMessageDialog(MainWindow.this,
                    "Parent Bundle for the language " + shortSfx + " does not" +
                    " exist.  RBEdit currently requires that you add it first",
                    "Parent Bundle Does Not Exist", JOptionPane.DEFAULT_OPTION);
                return;
            }
        }

/*
        // Create new resource bundle with specified extension
        for (int ndx = newBundles.size()-1; ndx >= 0; ndx--) {
            RBEdit.getRBEdit().getProject().addBundle(
                (ResBundle)newBundles.elementAt(ndx));
        }
*/
        Vector entries = null;
        proj.addBundle(new ResBundle(suffix, new File(filename), false, false));
    }

    /**
     Pops up a dialog to let the user decide whether to use a new root
     or import an existing one.  If using an existing one, opens up a file
     dialog so the user can select the existing root.

     PRE: none
     POST: may be a new project window in the main window.
    */
    public void doNewProject() {
        if (!doClose()) {
            return;
        }
        ResBundle bund = null;
        NewProjectDlg npDlg = new NewProjectDlg();

        npDlg.setVisible(true);

        if (npDlg.getResult() == npDlg.CANCELED) {
            return;
        } else if (npDlg.getResult() == npDlg.NEWROOT) {
            bund = new ResBundle("", new File("root"), false);
        } else {
            openDlg.setVisible(true);
            if (openDlg.getFile() == null) {
                return;
            }
            File open = new File(
                openDlg.getDirectory() + openDlg.getFile());

            bund = Project.loadBundle(open);
            if (bund == null) {
                return;
            }
            bund.setLooseFile(false);
            bund.setSuffix("");
        }

        NewBundleDlg nbDlg = new NewBundleDlg("", null, true);
        nbDlg.setTitle("Enter root's filename");
        nbDlg.setVisible(true);

        String name = nbDlg.getFilename();
        String fileType = nbDlg.getFileType();
        if (name == null) {
            return;
        }

        name = appendExtension(name, fileType);

        bund.setFile(new File(name));

        try {
            Project proj = new Project();
            proj.getConfig().fileType = fileType;
            RBEdit.getRBEdit().setProject(proj);
            RBEdit.getRBEdit().getProject().addBundle(bund);
            openProjectWindow(proj);
        } catch (PropertyVetoException ex) {
            ex.printStackTrace();
        }
    }

    /**
     Pops up a dialog to let the user select a project file.  The information
     in the project file is then used to load the project.

     PRE: none
     POST: may be a new project window in the main window.
    */
    public void doOpenProject() {
        if (!doClose()) {
            return;
        }

        openDlg.setVisible(true);
        if (openDlg.getFile() == null) {
            return;
        }
        File projFile = new File(openDlg.getDirectory() +
                openDlg.getFile());

        try {
            Project proj = new Project(projFile);
            RBEdit.getRBEdit().setProject(proj);
            openProjectWindow(proj);
        } catch (PropertyVetoException ex) {
            ex.printStackTrace();
        }
    }

    /**
     If an EditorWindow from a project is select, the project is saved.
     If a LooseFile EditorWindow is selected, that LooseFile is written out.

     PRE: none
     POST: project or loose file may be saved.
    */
    public void doSave() {
        JInternalFrame inf = desktop.getSelectedFrame();
        Project proj = RBEdit.getRBEdit().getProject();

        if (inf == null) {
            return;
        } else if (inf instanceof EditorWindow &&
                ((EditorWindow)inf).isLooseFile()) {
            Project.saveBundle(((EditorWindow)inf).getBundle());
        } else {
            if (proj.getProjectFile() == null) {
                doSaveAs();
            } else {
                proj.save(false);
            }
        }
    }


    /**
     Pops up a file dialog to allow the user to specify the new location of the
     project file.

     PRE: none
     POST: project may be saved.
    */
    public void doSaveAs() {
        if (RBEdit.getRBEdit().getProject() == null) {
            return;
        }

        saveDlg.setVisible(true);

        String filename = saveDlg.getFile();
        if (filename == null) {
            return;
        }

        if (!filename.toLowerCase().endsWith(".rbp")) {
            filename += ".rbp";
        }

        File file = new File(saveDlg.getDirectory() + filename);
        RBEdit.getRBEdit().getProject().setProjectFile(file);
        RBEdit.getRBEdit().getProject().save(true);
    }

    /**
     Pops up a save confirmation dlg if the project is dirty, then does as the
     user instructs (either cancels, exits, or saves and exits.
     @return false if the user wishes to cancel the exiting.

     PRE: project is open
     POST: project may be closed
    */
    public boolean doClose() {
        if (isDirty() && RBEdit.getRBEdit().getProject() != null) {
            int result;

            result = JOptionPane.showConfirmDialog(MainWindow.this,
                bundle.getString("closeProjWarningMessage"),
                bundle.getString("closeProjectDlg"),
                JOptionPane.YES_NO_CANCEL_OPTION);

            if (result == JOptionPane.CANCEL_OPTION) {
                return false;
            }

            if (result == JOptionPane.YES_OPTION) {
                Project proj = RBEdit.getRBEdit().getProject();

                if (proj.getProjectFile() == null) {
                    doSaveAs();
                } else {
                    proj.save(false);
                }
            }
            RBEdit.getRBEdit().setProject(null);

            for (int ndx = windows.size()-1; ndx >= 0; ndx--) {
                InternalWindow iw = (InternalWindow)windows.elementAt(ndx);
                removeWindow(iw);
                iw.dispose();
            }

            windows.removeAllElements();
        }
        return true;
    }


    /**
     Takes the filename, strips off a bad extension (if it has one), then
     puts on the correct extension
     @param file the filename to manipulate
     @param fileType name of the resource bundle file type (internationalized)

     PRE: file is not null.  fileType may be null
     POST: none
    */
    public String appendExtension(String file, String fileType) {
        String newFile = file;
        if (file.endsWith(".properties") || file.endsWith(".java") ||
                file.endsWith(".xml") || file.endsWith(".icu") ||
                file.endsWith(".")) {
            newFile = file.substring(0, file.lastIndexOf("."));
		}

        if (fileType == null || fileType.equals(bundle.getString("propertyFile"))) {
			newFile += ".properties";
		} else if (fileType.equals(bundle.getString("javaFile"))) {
			newFile += ".java";
		} else {
			throw new RuntimeException("Sorry, only java files and property" +
			        " files are currently supported: we don't support: " + fileType);
		}
        return newFile;
	}


    /**
     Opens the ProjectWindow.
     @param proj the project to be displayed in the project window

     PRE: none
     POST: the project is displayed
    */
    public void openProjectWindow(Project proj) throws PropertyVetoException {
        // INIT ProjectWindow with proj
        ProjectWindow projW = new ProjectWindow(proj);
        addTab(projW, true);
        editorPanel.add(projW);
    }

    /**
     Removes the JInternal Frame from Main Window
    */
    public void removeWindow(InternalWindow wind) {
        removeTab(wind);
        removeMenuItem(wind.getMenuItem());
        windows.removeElement(wind);
    }

    /**
     Removes the JInternal Frame from the eztabs and the parallel array
    */
    public void removeTab(InternalWindow wind) {
        // REMOVE frame from eztabs
        ezPane.remove(wind.getHiddenComponent());
    }

    /**
     Adds the JInternal Frame to both eztabs and the parallel vector
    */
    private void addTab(InternalWindow wnd, boolean insertAt0) {
        // IF insertAt0 THEN
        //     INSERT wnd into index 0 of eztabs
        // ELSE
        //     ADD wnd to end of eztabs

        if (insertAt0) {
            windows.insertElementAt(wnd, 0);
            windowMenu.add(wnd.getMenuItem());
            ezPane.insertTab(wnd.getTabName(), null, wnd.getHiddenComponent(),
                wnd.getToolTipText(), 0);
        } else {
            windows.addElement(wnd);
            windowMenu.add(wnd.getMenuItem());
            ezPane.addTab(wnd.getTabName(), null, wnd.getHiddenComponent(),
                wnd.getToolTipText());
        }
    }

    public void setSelectedTab(Component c) {
        ezPane.setSelectedComponent(c);
    }

    public void removeMenuItem(JMenuItem mi) {
        windowMenu.remove(mi);
    }

    /**
     Returns the JMenu for the 'Windows' menu
    */
    public JMenu getWindowMenu() {
        // RETURN windows menu
          return windowMenu;  // return windowMenu;
    }

    /**
     Allows the user to select a file, then returns a File with the path to the
     selected file.
     @param prompt The prompt to show to the user when a file selection dialog
     is displayed.
     @return a File pointing to the file selected by the user.

     PRE: prompt is not null
     POST: if user selects a file a File containing the file is returned
    */
    public File selectFile(String prompt) {
        // INIT file selection dialog with prompt
        // GET selected file
        // RETURN selected file
        return null;
    }

    public Dimension getPreferredSize() {
        return new Dimension(640,480);
    }

    public boolean isDirty() {
        return dirty;
    }

    public void setDirty(boolean modified) {
        dirty = modified;
    }
}
