package net.quartzware;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

/**
 NewBundleDlg for the RBEdit application.
 Creates a dialog box containing options to create a New Bundle.
 3 fields are available for user to define the language, country
 and variance of the new resource bundle.
 A new resource bundle is created when the user clicks the 'ok' button
 @author Chris Perluss
 @version 0.7.1
*/

// Version History
//     01/29/01 - Version 1.0 - Tony Johansen
//                Added the ability to add a new bundle(child).
//     11/30/00 - Version 0.9 - Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//     11/28/00 - version 0.7.1 - Susan Hunton
//                modified adt header, added pseudocode
//     11/20/00 - Version 0.6 - Anthony Tomarchio
//                Updated for internationalization
//     11/19/00 - Version 0.5 Albert revisions
//     11/17/00 - Version 0.3 Tony J and Chris
//                wrote PRE and POST conditions
//     11/15/00 - Version 0.2 Chris Perluss
//                wrote algorithm pseudocode
//     10/30/00 - Version 0.1 Chris Perluss
//                wrote ADT headers
public class NewBundleDlg extends CenterDlg {

    JComboBox languageComboBox = new JComboBox(Locale.getISOLanguages());
    JTextField languageField = new JTextField();

    JComboBox countryComboBox = new JComboBox(Locale.getISOCountries());
    JTextField countryField = new JTextField();

    JTextField variantField = new JTextField();
    JTextField filenameField = new JTextField();

    JComboBox fileTypeComboBox = new JComboBox();

    JLabel languageText = new JLabel();
    JLabel countryText = new JLabel();
    JLabel variantsText = new JLabel();
    JLabel filenameText = new JLabel();
    JLabel fileTypeText = new JLabel();

    JButton okButton = new JButton();
    JButton cancelButton = new JButton();

    String parSuffix = "";
    String suffix = null;
    String filename = null;
    String fileType = null;
    boolean getRootInfo = false;

    //Internationalization variables
    Locale currentLocale = Locale.getDefault();
    ResourceBundle bundle = ResourceBundle.getBundle("RBEdit", currentLocale);

    /**
     Constructs the NewBundleDlg
     PRE:  parent cannot be null
     POST:  dialog is created
    */
    public NewBundleDlg(Frame parent)    {
        super(parent);

        // CALL doTheLayout
        doTheLayout();

        // ADD Listeners
        addListeners();

        // DISPLAY the dialog
        pack();
    }

    /**
     Constructs the NewBundleDlg
     PRE:  none
     POST:  dialog is created
    */
    public NewBundleDlg()    {
        super((Frame)null);

        // CALL doTheLayout
        doTheLayout();

        // ADD Listeners
        addListeners();

        // DISPLAY the dialog
        pack();
    }

    /**
     Constructs the NewBundleDlg
     @param parentSfx the parent suffix to use to initialize the fields
     @param getRootInfo whether to lock this dialog into a mode where the user
            enters info about root.

     PRE:  parentSfx is not null
     POST: dialog is created
    */
    public NewBundleDlg(String parentSfx, String ext, boolean getRootInfo) {
        super((Frame)null);

        // if we are not getting root info, then ext cannot be null.
        //+ ASSERT((!getRootInfo && ext != null)) || (getRootInfo));

        parSuffix = parentSfx;
        this.getRootInfo = getRootInfo;
        fileType = ext;

        // CALL doTheLayout
        doTheLayout();

        // ADD Listeners
        addListeners();

        // DISPLAY the dialog
        pack();
    }

    /**
     Constructs the NewBundleDlg and sets the title equal to sTitle
     PRE:  sTitle cannot be null
     POST:  dialog is created
    */
    public NewBundleDlg(String sTitle) {
        this();

        setTitle(sTitle);
    }

    /**
     This function handles the layout of the New Bundle dialog box

     PRE: attributes have been set
     POST: GUI components have been added to the dlg and arranged appropriately.
    */
    private void doTheLayout() {
        setTitle(bundle.getString("newBundleDlg"));
        setResizable(false);
        setModal(true);
        getContentPane().setLayout(null);
        setSize(286, 232);

        languageComboBox.setBounds(96, 12, 100, 25);
        getContentPane().add(languageComboBox);

        languageField.setBounds(216, 12, 50, 25);
        getContentPane().add(languageField);

        countryComboBox.setBounds(96, 48, 100, 25);
        countryComboBox.insertItemAt("<None>", 0);
        countryComboBox.setSelectedIndex(0);
        getContentPane().add(countryComboBox);

        countryField.setBounds(216, 48, 50, 25);
        getContentPane().add(countryField);

        variantField.setBounds(96, 84, 170, 25);
        getContentPane().add(variantField);

        filenameField.setBounds(96, 120, 170, 25);
        getContentPane().add(filenameField);

        fileTypeComboBox.setBounds(136, 156, 130, 25);
        getContentPane().add(fileTypeComboBox);

        languageText.setText(bundle.getString("languageText"));
        languageText.setForeground(Color.black);
        languageText.setBounds(12, 12, 100, 25);
        getContentPane().add(languageText);

        countryText.setText(bundle.getString("countryText"));
        countryText.setForeground(Color.black);
        countryText.setBounds(12, 48, 100, 25);
        getContentPane().add(countryText);

        variantsText.setText(bundle.getString("variantsText"));
        variantsText.setForeground(Color.black);
        variantsText.setBounds(12,84,100,25);
        getContentPane().add(variantsText);

		filenameText.setText(bundle.getString("filenameText"));
		filenameText.setForeground(Color.black);
		filenameText.setBounds(12,120, 100, 25);
		getContentPane().add(filenameText);

        fileTypeText.setText(bundle.getString("fileTypeText"));
        fileTypeText.setForeground(Color.black);
        fileTypeText.setBounds(12, 156, 130, 25);
        getContentPane().add(fileTypeText);

        okButton.setText(bundle.getString("okButton"));
        okButton.setMnemonic((int)'O');
        okButton.setBounds(36, 192, 100, 25);
        getContentPane().add(okButton);

        cancelButton.setText(bundle.getString("cancelButton"));
        cancelButton.setMnemonic((int)'C');
        cancelButton.setBounds(156, 192, 100, 25);
        getContentPane().add(cancelButton);

        if (getRootInfo) {
            languageComboBox.removeAllItems();
            languageComboBox.addItem("Root");

            countryComboBox.setEnabled(false);

            languageField.setEnabled(false);
            languageField.setOpaque(false);

            countryField.setEnabled(false);
            countryField.setOpaque(false);

            variantField.setEnabled(false);
            variantField.setOpaque(false);

			fileTypeComboBox.addItem(bundle.getString("propertyFile"));
			fileTypeComboBox.addItem(bundle.getString("javaFile"));

            return;
		}

        fileTypeComboBox.addItem(fileType);
        fileTypeComboBox.setEnabled(false);

        // initialize values based on parent suffix
        if (parSuffix.equals("")) {
            countryComboBox.setSelectedIndex(0);
            return;
        }

        String sfx = parSuffix;
        if (sfx.indexOf('_') < 0) {
            languageComboBox.setSelectedItem(sfx);
            return;
        } else {
            languageComboBox.setSelectedItem(
                    sfx.substring(0, sfx.indexOf('_')));
            sfx = sfx.substring(sfx.indexOf('_')+1);
        }

        if (sfx.indexOf('_') < 0) {
            countryComboBox.setSelectedItem(sfx);
            return;
        } else {
            countryComboBox.setSelectedItem(
                    sfx.substring(0, sfx.indexOf('_')));
            sfx = sfx.substring(sfx.indexOf('_')+1);
        }
    }

    /**
     This function adds actionListeners to buttons which need to be watched.

     PRE: all GUI components have been initialized.
     POST: Listeners added to all componenets that need one.
    */
    private void addListeners() {

        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean addVariant = true;     //Flag to for whether or not
                                               //the user is adding a
                                               //variant to the sufix
               //Add Language code to fileType
                if (languageField.getText().equals("")) {
                    suffix = (String)languageComboBox.getSelectedItem();
                } else {
                    suffix = languageField.getText();
                }

               //Add Country code to fileType if specified
                if (!countryField.getText().equals("")) {
                    suffix = suffix + "_" + countryField.getText();
                } else if (countryComboBox.getSelectedIndex() != 0) {
                    suffix = suffix + "_" + countryComboBox.getSelectedItem();
                } else {
                    addVariant = false;
                }

                //Add Variant code to fileType if specified
                if (addVariant && !variantField.getText().equals("")) {
                    suffix = suffix + "_" + variantField.getText();
                }

                //Grab filename
                filename = filenameField.getText();
                if (filename.equals("")) {
                    filename = "NoName_" + suffix;
                }

                fileType = "" + fileTypeComboBox.getSelectedItem();

                //Close Dialog
                setVisible(false);
            }
        });

        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });
    }

    public void setVisible(boolean b) {

        if (b) {
            setLocation(50, 50);
        }
        super.setVisible(b);
    }

    public String getSuffix() {
        return suffix;
    }

    public String getFilename() {
        return filename;
    }

    public String getFileType() {
		return fileType;
	}
}