package net.quartzware;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import java.net.*;
import java.io.*;
import java.util.*;


/**
 NewProjectDlg
 Dialog box that asks the user if they want to use an existing root
 or if they want to create a new root in the creation of a new project.
 User can click on 'Use new root', 'Use existing root', or 'cancel'.
 @author Anthony Tomarchio
 @version 0.7.1
*/

// Version History
//     11/30/00 - Version 0.9   Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//     11/28/00 - version 0.7.1 - Susan Hunton
//                modified adt header, added pseudocode
//     11/20/00 - Version 0.6 - Anthony Tomarchio
//                Updated for internationalization
//     11/18/00 - Version 0.5 - Ben Leinweber
//                Added GUI code created by Visual Cafe
//                created Javadocs at the top
//                Formated code to be more readable
//     11/17/00 - Version 0.1 - Anthony Tomarchio
//                Created GUI code in visual cafe (in another file)

public class NewProjectDlg extends CenterDlg {
    public static final int CANCELED = 0;
    public static final int NEWROOT = 1;
    public static final int EXISTING = 2;

    JButton createNewRootButton = new JButton();
    JButton useExistingRootButton = new JButton();
    ImgPanel newProjImage = new ImgPanel();
    JButton cancelButton = new JButton();
    JTextArea newProjWarningMessage = new JTextArea();

    int result = CANCELED;

    //Internationalization variables
    Locale currentLocale = Locale.getDefault();
    ResourceBundle bundle = ResourceBundle.getBundle("RBEdit", currentLocale);


    /**
     Constructs the NewProjectDlg
     PRE: parent cannot be null
     POST:  dialog is created
    */
    public NewProjectDlg(Frame parent) {
        super(parent);

        // CALL doTheLayout
        doTheLayout();

        // ADD listeners
        doAddListeners();

        // DISPLAY the dialog
        pack();
    }

    /**
     Constructs the NewProjectDlg
     PRE: none
     POST:  dialog is created
    */
    public NewProjectDlg() {
        this((Frame)null);
    }

    /**
     Constructs the NewProjectDlg and sets the dialog title to sTitle
     PRE: sTitle cannot be null
     POST:  dialog is created
    */
    public NewProjectDlg(String sTitle) {
        this();

        setTitle(sTitle);
    }

    public Dimension getPreferredSize() {
        return new Dimension(440,147);
    }

    /**
     Sets the layout
     PRE: none
     POST:  dialog fields and buttons are arranged appropriatly
    */
    private void doTheLayout() {
        //setTitle("New Project");
        setTitle(bundle.getString("newProjectDlg"));
        setResizable(false);
        setModal(true);
        getContentPane().setLayout(null);

        //createNewRootButton.setText("Create New Root");
        createNewRootButton.setText(bundle.getString("createNewRootButton"));
        createNewRootButton.setActionCommand("Create New Root");
        createNewRootButton.setMnemonic((int)'N');
        createNewRootButton.setBounds(12,84,140,25);
        getContentPane().add(createNewRootButton);

        //useExistingRotButton.setText("Use Existing Root");
        useExistingRootButton.setText(bundle.getString("useExistingRootButton"));
        useExistingRootButton.setActionCommand("Use Existing Root");
        useExistingRootButton.setMnemonic((int)'U');
        useExistingRootButton.setBounds(168,84,140,25);
        getContentPane().add(useExistingRootButton);

        newProjImage.add(new ImageIcon(RBEdit.class.getResource(
            bundle.getString("newProjImage"))).getImage());
        newProjImage.setLayout(null);
        newProjImage.setBounds(12,12,80,65);
        getContentPane().add(newProjImage);

        cancelButton.setText(bundle.getString("cancelButton"));
        cancelButton.setMnemonic((int)'C');
        cancelButton.setBounds(324,84,100,25);
        getContentPane().add(cancelButton);

        newProjWarningMessage.setText(bundle.getString("newProjWarningMessage"));
        newProjWarningMessage.setOpaque(false);
        newProjWarningMessage.setEditable(false);
        newProjWarningMessage.setLineWrap(true);

        newProjWarningMessage.setFont(new Font("Dialog", Font.BOLD, 12));
        newProjWarningMessage.setBounds(108,24,275,40);
        getContentPane().add(newProjWarningMessage);
    }

    /**
     Adds necessary listeners

     PRE:  gui components have been initialized
     POST: gui componenets have listeners
    */
    public void doAddListeners() {
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                result = CANCELED;
                setVisible(false);
            }
        });
        createNewRootButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                result = NEWROOT;
                setVisible(false);
            }
        });

        useExistingRootButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                result = EXISTING;
                setVisible(false);
            }
        });
    }

    /**
     Returns the user's choice
     @return whether to start the new project from scratch or from an existing
             root.

     PRE:  gui components have been initialized
     POST: gui componenets have listeners
    */
    public int getResult() {
        return result;
    }
}