/* Generated by Together */

package net.quartzware;

import java.lang.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import javax.swing.JOptionPane;
import javax.swing.tree.*;
import javax.swing.event.*;

/**
 Project class
 Contains all the information about the currently open
 Project.  It holds all of the Resource Bundles and the current preferences
 for the project.
 @author Ben Leinweber
 @version 0.7.1
*/
// Version History
//     11/30/00 - Version 0.9   Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//     11/17/00 - Ver 0.3 Chris Perluss
//                Added PRE and POST conditions
//     11/15/00 - Ver 0.2 Ben Leinweber
//                Added algorithm pseudocode
//                Changed createBundle and importBundles from ResBundle's to void's
//                NOTE: ResBundles needs to do something with the Suffix!!!
//                NOTE: Look at save function
//     10/30/00 - Ver 0.1 Ben Leinweber
//                Initial Creation.  Javadoc code added.
public class Project implements TreeModel {
    /**
     * A list of all of the bundles loaded into the current project.  The bundle at
     * index 0 is the root bundle.  If the root bundle is not present, then index 0
     * is null.
     * @clientCardinality 1
     */
    private Vector bundles;

    /** A vector of listeners */
    private Vector listeners = new Vector();

    /** A path to the project file of the currently loaded project */
    private File projectFile;

    /** The current prefferences setting for the currently loaded project */
    private Config cfg;

    /** A Reader/Writer used to i/o PropertyFiles */
    private static JavaIO javaIO = new JavaIO();

    /** A Reader/Writer used to i/o PropertyFiles */
    private static PropertyIO propIO = new PropertyIO();

    /**
     This class encapsulates the project options
    */
    public class Config {

        // Entry color settings
        public Color changedFG;
        public Color changedBG;
        public Color unchangedFG;
        public Color unchangedBG;

        // Display/Write settings
        public boolean writeEscapeChar;
        public boolean listKeysAphabetically;
        public boolean displayKey;
        public boolean displayValue;
        public boolean displayParent;
        public boolean displayComments;

        // Resource file type for all bundles
        public String fileType;

        /**
         Default Constructor
        */
        public Config() {
            // Set the Default settings

            //  INIT Default Color Preferences
            unchangedFG = Color.black;
            unchangedBG = Color.white;
            changedFG = Color.gray;
            changedBG = Color.white;

            //  INIT Write Escape Characters flag
            writeEscapeChar = false;

            //  INIT List Keys Aphabetically flag
            listKeysAphabetically = true;

            //  INIT Display Key by Default flag
            displayKey = true;

            //  INIT Display Value by Default flag
            displayValue = true;

            //  INIT Display Parent Value by Default flag
            displayParent = true;

            //  INIT Display Comments by Default flag
            displayComments = true;

            fileType = null;
        }
    }

    /**
     Default constructor

     PRE: none
     POST: class is created
    */
    public Project() {
        // Set Default Values
        //  INIT bundles to default
        bundles = new Vector();

        //  INIT projectFile to null
        projectFile = null;

        //  INIT cfg to default
        cfg = new Config();
    }

    /**
     Creates a project using the given Config (preferences) and the given Vector
     of Resource Bundles.  Will be used mostly for creating a new project based
     on a Resource Bundle not made in RBEdit for a root.
     @param cfg Current preferences
     @param bundles a vector with Resource Bundles including a root

     PRE: cfg and bundles are not null
     POST: class is created
    */
    public Project(Config cfg, Vector bundles) {
        this.cfg = cfg;
        this.bundles = bundles;
        projectFile = null;
    }

    /**
     Loads a Project using a project file.  The project file lists the other
     files in the project, and this constructor tries to find them automatically
     @param f Project file selected from Open dialog box

     PRE: projectFile points to a file that can be used as a project file
     POST: the class is created
    */
    public Project(File projectFile) {
        bundles = new Vector();
        cfg = new Config();

        loadProjectFile(projectFile);
    }

    public void addTreeModelListener(TreeModelListener lis) {
        listeners.add(lis);
    }

    public Object getChild(Object parent, int index) {
        int count = index;
        String prSfx = ((ResBundle)parent).getSuffix();
        for (int ndx = 0; ndx < bundles.size(); ndx++) {
            ResBundle curBnd = (ResBundle)bundles.elementAt(ndx);
            if (curBnd.equals(parent)) {
                continue;
            }
            if (isChild(prSfx, curBnd.getSuffix())) {
                if (count-- <= 0) {
                    return curBnd;
                }
            }
        }
        return null;
    }

    public int getChildCount(Object parent) {
        int count = 0;
        String prSfx = ((ResBundle)parent).getSuffix();
        for (int ndx = 0; ndx < bundles.size(); ndx++) {
            ResBundle curBnd = (ResBundle)bundles.elementAt(ndx);
            if (curBnd.equals(parent)) {
                continue;
            }
            if (isChild(prSfx, curBnd.getSuffix())) {
                count++;
            }
        }
        return count;
    }

    public int getIndexOfChild(Object parent, Object child) {
        int count = 0;
        String prSfx = ((ResBundle)parent).getSuffix();
        for (int ndx = 0; ndx < bundles.size(); ndx++) {
            ResBundle curBnd = (ResBundle)bundles.elementAt(ndx);
            if (curBnd == parent) {
                continue;
            }
            if (isChild(prSfx, curBnd.getSuffix())) {
                if (curBnd != child) {
                    count++;
                } else {
                    break;
                }
            }
        }
        return count;
    }

    public Object getRoot() {
        return bundles.elementAt(0);
    }

    public boolean isLeaf(Object node) {
        String ndSfx = ((ResBundle)node).getSuffix();
        int count = 0;
        for (int ndx = 0; ndx < bundles.size(); ndx++) {
            ResBundle curBnd = (ResBundle)bundles.elementAt(ndx);
            if (curBnd == node) {
                continue;
            }
            if (isChild(ndSfx, curBnd.getSuffix())) {
                return false;
            }
        }
        return true;
    }

    public boolean isChild(String parSfx, String chldSfx) {
        int chldCnt = 0, parCnt = 0, ndx = 0;

        if (!chldSfx.startsWith(parSfx)) {
            return false;
        }

        for (ndx = 0; ndx < chldSfx.length(); ndx++) {
            if (chldSfx.charAt(ndx) == '_') {
                chldCnt++;
            }
        }

        for (ndx = 0; ndx < parSfx.length(); ndx++) {
            if (parSfx.charAt(ndx) == '_') {
                parCnt++;
            }
        }

        if (parSfx.equals("")) {
            return chldCnt == parCnt;
        } else {
            return chldCnt - parCnt == 1;
        }
    }

    public void removeTreeModelListener(TreeModelListener lis) {
        listeners.removeElement(lis);
    }

    public void valueForPathChanged(TreePath path, Object newValue) {
    }

    /**
     Gets the Root Resource Bundle
     @return the Root

     PRE: none
     POST: returns the root bundle if it is present, else it returns null
    */
    public ResBundle getRootBundle() {
        return (ResBundle)bundles.elementAt(0);
    }

    /**
     Deletes the Entries in all of the Resource Bundles that match the given key
     @param key The Key to be deleted

     PRE: key is not null
     POST: no Entry containing the given key will be in the project.
    */
    public void deleteKey(String key) {
        // FOR EACH ResBundle in bundles DO
        //     Delete key from Current ResBundle
        // NEXT ResBundle

        for (int ndx = 0; ndx < bundles.size(); ndx++) {
            ((ResBundle)bundles.elementAt(ndx)).removeEntry(key);
        }
    }

    /**
     Returns a File which contains the path to the Project File.
     @return the ProjectFile
    */
    public File getProjectFile() {
        return projectFile;
    }

    /**
     Changes the File used to write out.  In other words, it changes where the
     project file is written to.
     @param projectFile The new ProjectFile
    */
    public void setProjectFile(File projFile) {
        projectFile = projFile;

        for (int ndx = 0; ndx < bundles.size(); ndx++) {
            ResBundle bnd = (ResBundle)bundles.elementAt(ndx);
            bnd.setFile(new File(projFile.getParent() + File.separator +
                    bnd.getFile().getName()));
       }
    }

    /**
     Adds the given Resource Bundle to the current Project
     @param rb The resource bundle to be added

     PRE: rb is not null
     POST: the resource bundle is added to the project
    */
    public void addBundle(ResBundle rb) {
        int newNdx, ndx;
        Vector newEnts;
        Vector entries = rb.getAllEntries();
        String suffix = rb.getSuffix();

        if (projectFile != null) {
            rb.setFile(new File(projectFile.getParent() + File.separator +
                    rb.getFile().getName()));
        }

        if (suffix.equals("")) {
            newEnts = new Vector();
        } else if (suffix.lastIndexOf('_') < 0) {
            newEnts = getRootBundle().getAllEntries();
        } else {
            newEnts = getBundle(suffix.substring(
                    0, suffix.lastIndexOf('_'))).getAllEntries();
        }


        // Add inherited keys from the parent
        for (newNdx = 0; newNdx < newEnts.size(); newNdx++) {
            String key = ((Entry)newEnts.elementAt(newNdx)).getKey();
            for (ndx = 0; ndx < entries.size(); ndx++) {
                if (((Entry)entries.elementAt(ndx)).getKey().equals(key)) {
                    break;
                }
            }
            if (ndx == entries.size()) {
                try {
                    Entry ent = (Entry)((Entry)newEnts.elementAt(newNdx)).clone();
                    ent.setOwner(rb);
                    ent.setUnique(false);
                    rb.addEntry(ent);
                } catch (CloneNotSupportedException ex) {}
            }
        }

        if (rb.getSuffix().equals("")) {
            bundles.insertElementAt(rb, 0);
        } else {
            bundles.addElement(rb);
        }
        fireUpdate();
    }

    /**
     Removes the given Resource Bundle from the current Project
     @param rb The resource bundle to be removed

     PRE: rb is not null
     POST: the resource bundle is removed if it exists
    */
    public void removeBundle(ResBundle rb) {
        // Call removeElement
        bundles.removeElement(rb);

        fireUpdate();
    }

    /**
     Removes the Resource Bundle that matches the given suffix and all of its
     children from the current Project.
     @param rb The resource bundle to be removed

     PRE: none
     POST: the resource bundles are removed
    */
    public void removeBundleAndChildren(String sfx) {
        for (int ndx = bundles.size()-1; ndx >= 0; ndx--) {
            ResBundle curBund = (ResBundle)bundles.elementAt(ndx);
            if (curBund.getSuffix().startsWith(sfx)) {
                bundles.removeElementAt(ndx);
            }
        }
        fireUpdate();
    }

    /**
     Saves all of the currently dirty Resource Bundles.  Does not write any
     files that have not been changed, unless the saveAs flag is true.  If the
     saveAs flag is true, everything is written out.
     @param saveAs Save As Flag (described above)

     PRE: none
     POST: the project is saved.
    */
    public void save(boolean saveAs) {
        for (int ndx = 0; ndx < bundles.size(); ndx++) {
            if (((ResBundle)bundles.elementAt(ndx)).isDirty() || saveAs) {
                saveBundle((ResBundle)bundles.elementAt(ndx));
            }
        }
        saveProjectFile();
    }

    /**
     Saves the specified ResBundle.  Will not pop up a dialog
     of any kind.
     @param bundle bundle to write out.

     PRE: none
     POST: the project is saved.
    */
    public static void saveBundle(ResBundle bund) {
        if (bund.getFile().getName().toLowerCase().endsWith(".java")) {
            javaIO.writeBundle(bund, bund.getFile());
        } else {
            propIO.writeBundle(bund, bund.getFile());
        }
    }

    /**
     Save out the project file

     PRE: none
     POST: the project file is saved.
    */
    public void saveProjectFile() {
        Properties props = new Properties();

        props.setProperty("bundleCount", "" + bundles.size());

        for (int ndx = 0; ndx < bundles.size(); ndx++) {
            ResBundle bnd = (ResBundle)bundles.elementAt(ndx);
            props.setProperty("bundleFile" + ndx, bnd.getFile().getName());
            props.setProperty("bundleSuffix" + ndx, bnd.getSuffix());
        }

        props.setProperty("unchFG", "" + cfg.unchangedFG.getRGB());
        props.setProperty("unchBG", "" + cfg.unchangedBG.getRGB());
        props.setProperty("chFG",   "" + cfg.changedFG.getRGB());
        props.setProperty("chBG",   "" + cfg.changedBG.getRGB());

        props.setProperty("writeEscapeChar", "" + cfg.writeEscapeChar);
        props.setProperty("listKeysAphabetically", "" + cfg.listKeysAphabetically);
        props.setProperty("displayKey", "" + cfg.displayKey);
        props.setProperty("displayValue", "" + cfg.displayValue);
        props.setProperty("displayParent", "" + cfg.displayParent);
        props.setProperty("displayComments", "" + cfg.displayComments);

        try {
            FileOutputStream out = new FileOutputStream(
                projectFile.getPath());

            props.store(out, "RBEdit Property File");

            out.flush();
            out.close();
        } catch (IOException ex) {ex.printStackTrace();}
    }

    /**
     Load in a project file

     PRE: none
     POST: a new project is created
    */
    public void loadProjectFile(File projFile) {
        projectFile = projFile;

        Properties props = new Properties();
        try {
            FileInputStream in = new FileInputStream(projFile.getPath());
            props.load(in);
        } catch (IOException ex) {ex.printStackTrace();}

        int count = Integer.parseInt(props.getProperty("bundleCount"));
        for (int ndx = 0; ndx < count; ndx++) {
            String fileName = props.getProperty("bundleFile" + ndx);
            String suffix = props.getProperty("bundleSuffix" + ndx);

            ResBundle bund = loadBundle(new File(projFile.getParent() +
                    File.separator + fileName));
            bund.setSuffix(suffix);
            addBundle(bund);
        }

        cfg.unchangedFG = new Color(Integer.parseInt(props.getProperty("unchFG")));
        cfg.unchangedBG = new Color(Integer.parseInt(props.getProperty("unchBG")));
        cfg.changedFG = new Color(Integer.parseInt(props.getProperty("chFG")));
        cfg.changedBG = new Color(Integer.parseInt(props.getProperty("chBG")));

        cfg.writeEscapeChar =
                props.getProperty("writeEscapeChar").equals("true");

        cfg.listKeysAphabetically =
                props.getProperty("listKeysAphabetically").equals("true");

        cfg.displayKey = props.getProperty("displayKey").equals("true");
        cfg.displayValue = props.getProperty("displayValue").equals("true");
        cfg.displayParent = props.getProperty("displayParent").equals("true");

        cfg.displayComments =
                props.getProperty("displayComments").equals("true");
    }

    /**
     Loads all bundles listed in the project file at the given path.
     @param file Where the project file is.

     PRE: none
     POST: the project is loaded.
    */
    public static Project load(File file) {
        return new Project();
    }

    /**
     Loads the specified ResBundle.
     @param File the file to read in from.

     PRE: none
     POST: none
    */
    public static ResBundle loadBundle(File loc) {
        try {
            String name = loc.getName().toLowerCase();
            if (name.endsWith(".properties")) {
                return propIO.readBundle(loc);
            } else if (name.endsWith(".java")) {
                return javaIO.readBundle(loc);
            } else if (name.endsWith(".xml")) {
                throw new Exception("Sorry, XML files are not supported yet");
            } else if (name.endsWith(".icu")) {
                throw new Exception("Sorry, ICU files are not supported yet");
            } else {
                throw new Exception("Sorry, file type unrecognized");
            }
        }
        catch(Exception ex) {
            JOptionPane.showMessageDialog(null, ex.getMessage(), "Warning",
                JOptionPane.ERROR_MESSAGE);
        }
        return null;
    }

    /**
     Gets the bundle that matches the suffix
     @return the bundle which matches the suffix

     PRE: suffix is not null
     POST: the bundle which matches the suffix is returned if found
    */
    public ResBundle getBundle(String suffix) {
        for (int ndx = 0; ndx < bundles.size(); ndx++) {
            if (((ResBundle)bundles.elementAt(ndx)).getSuffix().equals(suffix)) {
                    return (ResBundle)bundles.elementAt(ndx);
            }
        }

        return null;
    }

    /**
     Gets all of the Resoruce Bundles
     @return all of the Resource Bundles in a Vectorj
    */
    public Vector getAllBundles() {
        return bundles;
    }

    /**
     Adds a new key to the root.  The Bundleren will then be able to override
     this new key
     @param key the name of the key to be added

     PRE: key is not null
     POST: key is added to root bundle
    */
    public void addKey(String key) {
        // IF key does not exist in root THEN
        //     Create new Entry with given key
        //     Add key to Root Bundle
        // END IF
    }

    public void fireUpdate() {
        TreeModelEvent tme = new TreeModelEvent(getRoot(),
                new TreePath(getRoot()));

        for (int ndx = 0; ndx < listeners.size(); ndx++) {
            ((TreeModelListener)listeners.elementAt(ndx)).treeStructureChanged(tme);
        }
    }

    /**
     Returns a Config containing all of the options for the current Project.
    */
    public Config getConfig() {
        return cfg;
    }
}
