/* Generated by Together */

package net.quartzware;

import java.net.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import javax.swing.*;

/**
 PropertyIO is a class which is able to both read to and from property files.
 @author Chris Perluss
 @version 0.1
*/

// Version History
//   2/06/01 - Version 0.1 Chris Perluss
//             Creation

public class PropertyIO implements RBReader, RBWriter {

    /**
     Reads the File pointed to by file and returns a ResBundle
     @param file Where to read this bundle from
     @return a ResBundle created from the file pointed to by file
    */
    public ResBundle readBundle(File file) {
        ResBundle bundle = new ResBundle("", file, false);
        try {
            BufferedReader in = new BufferedReader(new
                    FileReader(file.getPath()));
            Vector entries;
            String lastCom = new String("");
            String endLine = System.getProperty("line.separator");

            while (in.ready()) {
                String key;
                String val;
                String line = in.readLine();

                if (line == null) {
					break;
				}

                if (line.equals("")) {
                    continue;
                } else if (line.trim().startsWith("/")) {
                    for (int ndx = 0; ndx < line.length(); ndx++) {
                        char chr = line.charAt(ndx);
                        if (Character.isLetterOrDigit(chr)) {
                            lastCom = line.substring(ndx);     // Has char other
                            break;                        // than banner chars
                                             // (/ and *) must be a real comment
                        }
                    }
                } else if (line.indexOf('=') >= 0) {
                    key = line.substring(0, line.indexOf('=')-1);
                    val = line.substring(line.indexOf('=')+1, line.length());
                    Entry ent = new Entry(bundle, key, new StringVal(val),
                        lastCom, true);
                    bundle.addEntry(ent);
                    lastCom = new String("");
                } else {
                    System.out.println("bad line = \"" + line + "\"");
                    throw new Exception("Error Reading.  Unparseable text");
                }
            }
        } catch (FileNotFoundException ex) {
                JOptionPane.showMessageDialog(null, "Could not find file ",
                "Error", JOptionPane.ERROR_MESSAGE);
        } catch (Exception ex) {
                JOptionPane.showMessageDialog(null, "Sorry, only Strings are " +
                "currently supported", "Error", JOptionPane.ERROR_MESSAGE);
        }
        return bundle;
    }

    /**
     Writes the resource bundle out to the file
     @param rb the bundle to write out.
     @param file where to write out the bundle to.
    */
    public void writeBundle(ResBundle rb, File file) {
        try {
            BufferedWriter out = new BufferedWriter(new
                    FileWriter(file.getPath()));
            Vector entries = rb.getAllEntries();
            for (int ndx = 0; ndx < entries.size(); ndx++) {
                Entry ent = (Entry)entries.elementAt(ndx);

                if (!ent.isUnique()) {
					continue;
				}

                if (!(ent.getValue() instanceof StringVal)) {
                   throw new Exception("");
                }
                String com = ent.getComment();

                if (!com.equals("")) {
                    out.newLine();
                    out.write("// " + com, 0, com.length() + 3);
                    out.newLine();
                }

                out.write("" + ent.getKey() + " = " + ent.getValue(),
                    0, ent.getKey().length() + ("" + ent.getValue()).length()
                    + 3);
                out.newLine();
            }
            out.flush();
            out.close();
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(null, "Error saving to file ",
                "Error", JOptionPane.ERROR_MESSAGE);
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(null, "Sorry, only Strings are " +
            "currently supported", "Error", JOptionPane.ERROR_MESSAGE);
        }
    }
}
