/* Generated by Together */

package net.quartzware;
import javax.swing.*;
import java.lang.*;
import java.util.*;

/**
QUARTZWARE RBEdit

Created By: Susan Hunton, Tony Johanson, Albert Lee
        Ben Leinweber, Chris Perluss, and Tony Tomarchio

VERSION: 0.9
November 10, 2000


SOFTWARE ARCHITECTURE: model/view


DESIGN OVERVIEW

RBEdit uses an Object Oriented Design and a Model/View/Controller architecture.
The main window controls display of the main window and handles most of the user
interaction.  Various functionality is, of course, also in the other classes:

GUI:
  EditorWindow:  draws the entries from a resource bundle and handles any user
                    interaction which occurs within its border.
  ProjectWindow: displays all resource bundles available in the current project
                    in a tree format.
Domain Models:
  Entry:       encapsulates an entry-- one key, one value, and one comment.
  Project:     contains a list of all resource bundles in the project, as well
                  as any options the user may change.
  ResBundle:   encapsulates the notion of a 'Resource Bundle.'  It contains a
                  vector of entries, as well as a language, country, etc.

  Value:        Abstract base class which has the following derived classes:
  BinaryPath:   contains a path to a binary file.  Stored as a String.
  BinaryValue:  contains a binary number of unspecified length.  Stored as a
                   vector of bytes.
  IntArray:     contains an array of integers.  Stored as a vector of Integers.
  IntegerVal:   an integer, stored inside of the Integer class.
  Message:      A complex message, possibly containing choices.  Stored as a
                   String.
  ObjectVal:    A java object.  The parameters to use in the constructor are
                   stored as a string.
  StringArray:  An array of Strings.  stored as a vector of Strings.
  StringVal:    A String.  Stored using the java String class.
  TaggedArray:  Two parallel array using the entries of one as indecies for the
                   other.

IO:
  RBReader:    interface which is used to create classes which can read resource
                  bundles from one of the various formats and convert it to a
                  ResBundle.
  RBWriter:    interface which is used to create classes which can write
                  resource bundles in one of the various formats from a
                  ResBundle.


The main functionality of our program, of course, is allowing the user to modify
the different data types.  For this, polymorphism seemed the best option.  All
of the data types are derived from one class, Value, which has an abstract
function which edits the value.  Each of the derived classes defines this
function with code that properly edits themselves.

The EditorWindows use a JTable.  Instead of using a default TableModel, we
create our own.  This way, our data can be directly accessed by the JTable
without having to arrange our data into a form digestible by the
DefaultTableModel.

The EditorWindows also uses a custom CellEditor, so that proper editing will
occur.  When a cell in the JTable is double-clicked, the JTable sends a
message to its CellEditor to see what to do.  Since there is a custom
CellEditor, editValue is called on whichever Value happens to be in that cell.
If true is returned from the editValue call, then the value is a simple one and
wishes that we edit it in-label.  If false is returned, the the Value is going
to pop up a dialog and therefor does not need in-label editing.

The ProjectWindow uses a JTree.  Instead of using the default TreeModel, we will
create our own TreeModel.  This way the JTree can work directly from our data
without having to arrange our data into a form digestible by the
DefaultTreeModel.




DESIGN ISSUES

Project File:

Issue:     When a child is added or removed from the project file, when is the
              project file updated?
Decision:  The project file is saved instantly, in order to minimize
              de-synchronization between children and the root.
Alternatives:
           To save when the user saves the project.

Loose Files:

Issue:     Whether to display loose files in the project window.
Decision:  No, just open them up in an Editor Window.  If it is shown in the
              project window, some confusion may ensue as to which files are
              in the project and which are loose files.  Also, this would
              raise confusion as to which files are saved when the project
              is saved. To avoid these extra problems, we decided to not put
              loose files in the project window.
Alternatives:
           -Display loose files in the project window.

New Project:

Issue:     Whether to allow the user to import an existing root.
Decision:  We will allow the user to import an existing root.  We thought it
              would be useful enough to the client to justify its existence.
              This feature will most probably be re-evaluated during the scope
              evaluation process in the latter half of the development phase.
Alternatives:
           -Restrict the user to using roots created by RBEdit.

Issue:     What to do if the root file being read is not parseable.
Decision:  Pop up a message box informing the user of the error.  Subsequently,
              we will refuse to load the file.  User-assisted import
              functionality would probably  be quite complex and is far beyond
              the scope of this development project.
Alternatives:
           -Refuse to read the file.
           -Ignore the unparseable text.
           -Save the unparseable text and write it out to the same place when
              saving.
           -Implement 'user assisted importing', where the user looks at the
              file and tells the computer what each part is.

Issue:     In the Open Project Dialog, should we create our own icon?
Decision:  Albert felt that the default windows icon seemed to intimidating, so
              he decided to create his own.  Albert felt sure he could create a
              new icon which was more cute, cuddly and friendly.
Alternatives:
           -Use the default windows icon.

Issue:     Whether to allow addition, removal, and renaming of Key Names when
              the root is not present.
Decision:  Because this would result in the de-synchronization between the root
              and the children, we decided not to allow it.  Bundleren may still
              get out of sync from the root through other means, including hand
              editing of the files, so support will be added to detect
              de-synchronization.
Alternatives:
           -Allow the user to add, remove, and rename Key Names when the root is
              not present.

Saving Files:

Issue:     What save-related menus items should we add to the File menu, and
              what should they save.
Decision:  We decided to keep things simple and only use 'Save' and 'Save
              Project As'.  If the EditorWindow for a loose file is in focus
              when 'Save' is selected, then only the loose file displayed by
              that EditorWindow is saved.  If any other window is in focus when
              'Save' is selected, then the entire project is saved.  'Save
              Project As' allows the user to save a copy of the project under a
              different file name.
Alternatives:
           -Save Project, Save As, Save All

Importing Files:

Issue:     Whether to allow the user to import existing resource bundles.
Decision:  We decided to allow the user to import existing resource bundles,
              even though the client said that it was not required
              functionality.  We thought it would be useful enough to justify
              its existence.  This feature will most probably be re-evaluated
              during the scope evaluation process in the latter half of the
              development phase.
Alternatives:
           -Limit the user to only use resource bundles created by RBEdit.

Issue:     When importing a child, what to do if the child has keys that are not
              in the root.
Decision:  We discussed the possibility of just adding these keys to the root,
              since the user could then delete the ones he/she doesn't want, but
              then we decided it would be much more user friendly to pop up a
              dialog and ask the user what he/she wants to do.  The options are
              'Add key to root' and 'Remove key from child'.  We don't allow the
              user to put a key in the child that's not in the root because the
              Client expressly forbade it.
Alternatives:
           -Automatically: add the key to root, delete the key from the child,
              leave the key in the child but not in the root, or cancel loading.

Issue:     When the user selects the 'Remove Bundle' option, should the resource
              bundle file itself be deleted, or should it just be removed from
              the project file?
Decision:  We decided to just remove the child from the project.  If we
              automatically deleted the file, but the user wanted to keep it,
              then that would result in a nasty problem.  On the other hand, if
              we don't delete the file, the user can delete the file at his/her
              own discretion.
Alternatives:
           -Delete the file off of the hard drive.

Issue:     When removing a parent from the project, should its children be
              removed as well?
Decision:  Yes, we decided to remove the children as well, since that made the
              most logical sense.
Alternatives:
           -Remove the children from the project.

Working with Entries:

Issue:     Should there be multiple ways to delete a key?
Decision:  Yes, this gives the user much more freedom and makes the UI more
              consistent with standard UI's.  There are three ways to delete a
              key and the user can choose from any of the three.
Alternatives:
           -Only allow the user to delete keys through the menu system or a tool
              bar.  We decided against this because it limits the usability.

Issue:     Should the user be able to rename a key?
Decision:  Yes they should, since it's easier than deleting a key, adding a new
              one and then retyping all of the entries.
Alternatives:
           -Do not allow the user to rename a key.  If a key were misspelled,
              the user would have to delete it and make a new one.  This is too
              much work.

Issue:     Where can the user be able to rename a key?
Decision:  A key can be renamed from any editor window.  A warning message
              will pop up every time a user tries to rename a key to inform him
              that .
Alternatives:
           -We considered only allowing the user to rename a key from the root,
              but decided that this may not be intuitive to many users.

Issue:     What should be done when a value is changed from its parent?
Decision:  The background color should be changed to light gray or beige for the
              entry to make it easier to identify which values have been changed
              and which haven't.
Alternatives:
           -We discussed putting an icon or a check mark next to the value to
              specify it had been changed.
           -We could have allowed the option of only showing the changed, or
              unchanged values.

Issue:     How do you edit an entry?
Decision:  To handle a simple entry, we decided the easiest way was to simply
              double click in the field and allow the user to edit it like a
              spreadsheet document.  More complicated data types required a more
              complicated way to make an entry, so we have dialog boxes for
              them.  We also have a menu item to edit the selected key.
Alternatives:
           -Use dialog boxes to modify all data types.

Searching:

Issue:     How should the user be allowed to search bundles?
Decision:  We decided to model our search engine after Visual Studio's search
              engine.  This allows us to use our Editor Window for searching so
              we don't have to design a completely new 'results' window.  The
              user can search a single file or the entire project if they wish.
Alternatives:
           -To create a new window that contained all of the entries that
              matched the search results.  This allows the user to see all of
              the search results in one place, rather than having to push F3
              repeatedly to see all the results.

Easy Tabs:

Issue:     How should we allow a user to quickly navigate through the open
              windows?
Decision:  We came up with the idea for Easy Tabs based on a modification that
              Chris has for Visual Studio.  The easy tabs are fairly similar to
              the windows task bar and allow quick navigation of the windows.
Alternatives:
           -By pressing Ctrl-Tab.
           -By selecting a window from the 'Windows' menu.

String Array:

Issue:     How should a user be able to add and remove entries in a StringArray?
Decision:  The user may only add and remove entries in a String Array from the
              root dialog box.  This is to prevent misunderstanding what adding
              and removing does.  Adding and removing effects all of the String
              Arrays of a specific key in every resource bundle.
Alternatives:
           -Allow a user to add or remove a string array from any entry in any
              bundle.  This would make the files go out of sync though, so it's
              not a viable alternative.

Issue:     Should the parent be shown in the String Array dialog box?
Decision:  Yes, this conforms more closely to the standard set by our
              EditorWindow.
Alternatives:
           -At first we thought that we wouldn't really need it, but after
              talking to George, our opinions changed.

Project Window:

Issue:     How to display list of resource bundles?
Decision:  Display list in a tree format as a "project" window.  The tree allows
              easy viewing of the bundle hierarchy and easy movement between
              bundles because the user simply needs to select the bundle to work
              with and then click on the bundle.
Alternatives:
           -Display list of bundles in a simple list rather than in a tree or to
              not have list at all.

Editor Window:

Issue:     How to display resource bundles?
Decision:  The "editor" window.  In the editor window we have included four
              columns: key, value, parent value and comments.  This is to make
              it easy to compare the differences between the values of parent
              and child for a particular key.  We wanted to have all information
              for a particular key on one line so as to be easily read, compared
              and modified.  The editor window with four columns allows the user
              to get all the necessary information in one line rather than
              having to go to multiple windows or files.
Alternatives:
           -We could have chosen to only show three columns and to omit the root
              key value but we wanted to have easy comparison and felt it
              provided convenience to the user.

Issue:     How to compare resource bundles?
Decision:  Allow multiple editor windows to be open simultaneously.  We decided
              that just as Microsoft Word allows many documents to be open
              simultaneously, we would stay with their standard and allow the
              user determine how many windows are manageable to have open at one
              time.
Alternatives:
           -We could have restricted the number of windows that can be opened to
              two, three, or four windows but we did not want to restrict our
              users.

Editing:

Issue:     Should the user be able to undo/redo?
Decision:  We have decided to include the option to undo and redo the last
              change.  We included this feature to make the application more
              user-friendly and easier to make changes.  This function is also
              standard on other applications so we wanted to keep with the
              standard.  We limited the undo/redo to the last change to keep the
              programming simple and to reduce the program's record keeping.
Alternatives:
           -Not to allow any undo/redo or to allow unlimited undo/redo.

Issue:     Should short cuts be built in for various operations?
Decision:  In many of our functions, we have outlined multiple ways of
              performing the function: through the menu, right clicking, or
              icons.  We decided on this so that new users can use menus to find
              tools but more advanced users can make changes more quickly by
              using shortcuts.
Alternatives:
           -Make users go through menus for all functions.

Testing:

Issue:     Which data types need to be testable?
Decision:  Messages and Object values need to be testable.  Having other data
              types testable would be overkill.  String values do not need to be
              tested.
Alternatives:
           -All data types are testable, or none are.

Preferences:

Issue:     How should information be presented so that important items are
              highlighted?
Decision:  Show differences or similarities between parent and child values by
              using color. The entire line where a root and child key are the
              same/different will have a colored background based on user
              preferences. We chose to color the entire line because it would
              be easy to distinguish the differences. User can choose from a
              variety of background colors.
Alternatives:
           -Highlight lines where the root and child keys are the same.
           -Highlight lines where the root and child keys are different.
           -Highlight differences/similarities with bold.
           -Highlight differences/similarities with different font colors such
              as gray/black or black/red.
           -Highlight differences/similarities with colors decided by the
              designers.

Issue:     How should resource bundles be displayed in the project window?
Decision:  The project tree, can show full resource bundle name or only the name
              of the language and region so that user can see only location or
              can see the name of the file, depending on the needs of the user.
              We did this to allow different types of information to be
              displayed depending on the user.  A less experienced user may not
              want to deal with the full file name, and just want to know the
              language and location.
Alternatives:
           -Allow the user only one way of displaying the resource bundle.

Issue:     How should multiple windows be organized and shown to the user?
Decision:  Give the user the option to tile windows horizontally, vertically,
              and cascaded.  This is so that the user can easily manage multiple
              open editor windows in the format that they find most convenient
              and efficient.
Alternatives:
           -No window organization.
           -Offer only one type of window organization.

Issue:     Should we include Cut, Copy, Paste features.
Decision:  Since one of our requirements of this application was to make it
              easy to use, we tried to make our application as standard as
              possible by including Cut, copy, and paste features. This can
              increase productivity and thus usability.
Alternatives:
           -Not to allow these options.



TOOLS

Sun's Java 2 SDK, Standard Edition Version 1.3
Together Control Center, Version 4.1 Build 1015
Adobe Photoshop, Version 5.0
Microsoft Paint, Version 98
Microsoft Word 2000, Version 9.0.4119 SR-1



LIBRARIES

java.awt:          Contains drawing classes needed when interfacing with java
                      drawing methods.
java.awt.event:    Contains listener classes needed to perform basic
                      listening requirements.
java.io:           Has necessary Input/Output classes, such as File.
java.lang:         Offers core functionality for java programs.  All classes
                      should import this.
java.net:          Has the File class, which is more versatile than the File
                      class.
java.util:         Has the Vector class, which offers more functionality than
                      an ordinary array.
javax.swing:       Contains Swing classes used for drawing.
javax.swing.table: Contains model and listener classes needed for using advanced
                      JTable features.
javax.swing.event: Contains event handling classes for the new GUI elements
                      introduced in Swing.



EXTERNAL REFERENCES

This document describes the location of various project documents.

* User Manual - http://www.quartzware.net/technical/usermanual.html
Note: All versions are stored in Microsoft Word format

* JavaDocs - http://www.quartzware.net/documents/javadoc/
Note: The javadocs are in html format and are accessible
from any web browser.

* User Interface Prototype Screenshots - http://www.quartzware.net/technical/uiprototype.html
Note: The Screentshots are in .gif format. They will soon be integrated
into the user manual with more detailed descriptions.

* Class Interaction Diagrams (Use Cases) - http://www.quartzware.net/technical/usecases.html
Note: The Interaction diagrams are in .jpg format

* QuartzWare Team Schedule -
http://www.quartzware.net/planning/softprojhist.html
Note: The schedule is in HTML format.

* QA Checklist - http://www.quartzware.net/planning/qaplan.html
Note: The checklist is in HTML format

Version History:
11/10/00 - Tony Tomarchio - Document Creation
*/

/**
 Main module for the RBEdit application.
 @author Chris Perluss
 @version 0.7.1
*/

// Version History
//     11/30/00 - Version 0.9   Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//     11/28/00 - Version 0.7 Chris Perluss
//                Added command line parameter hack to show otherwise
//                inaccessible dialogs.
//     11/17/00 - Version 0.3 Chris Perluss & Albert Lee
//                Removed '!=' from pseudocode
//     11/16/00 - Version 0.3 Ben Leinweber
//                Removed '!=' from pseudocode
//     11/15/00 - Version 0.2 - Anthony Tomarchio
//                Added algorithm pseudocode
//     10/30/00 - Version 0.1 - Anthony Tomarchio
//                Initial Creation. JavaDocs created.

public class RBEdit {

    //Internationalization variables
    Locale currentLocale = Locale.getDefault();
    ResourceBundle bundle = ResourceBundle.getBundle("RBEdit", currentLocale);

    /**
     Default Constructor

     PRE: none
     POST: class is created
    */
    public RBEdit() {
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);

        // INIT mainWindow using this RBEdit.
        mainWindow = new MainWindow(this);
        mainWindow.setIconImage(new ImageIcon(RBEdit.class.getResource(bundle.getString("minimizeAppIcon"))).getImage());
    }

    /**
     Java calls this static function to start the program.  After it is called,
     it creates an instance of this class.
     @param args command line arguments, if any.

     PRE: none
     POST: a new RBEdit is created
    */
    public static void main(String[] args) {
        if (args.length > 0) {
            new KeyDoesNotExistDlg();
            new RemoveBundleDlg();
            new RenameKeyDlg();
            System.exit(0);
        }
        else {
            // INIT rbedit
            rbedit = new RBEdit();
        }
    }

    /**
     Sets the Project class that contains the project file and options for the
     current session of RBEdit.

     @return The Project.
    */
    public void setProject(Project proj) {
        project = proj;
    }

    /**
     Returns the Project class that contains the project file and options for the current session of RBEdit.
     @return The Project.
    */
    public Project getProject() {
        return project;
    }

    /**
     Returns the Window class which draws this RBEdit.
     @return The MainWindow.
    */
    public MainWindow getMainWindow() {
        return mainWindow;
    }

    /**
     Allows any class in the RBEdit application to get instant access to the
     Main module by calling 'RBEdit.getApp()'.
     @return The MainWindow which draws this RBEdit.
    */
    public static RBEdit getRBEdit() {
        return rbedit;
    }

    /**
     Displays a dialog to let the user enter options about a new project, then
     it creates the new project.
     @return true if we are importing an existing root. false if we are starting
        from scratch.

     PRE: none
     POST: returns whether we are using an existing root.
    */
    public boolean showNewProjectDlg() {
        // DISPLAY new project dialog
        // GET and RETURN 'use existing root'
        return false;
    }

    /**
     Called when the user wants to add a new key

     PRE: none
     POST: new key may be added to the root
    */
    public void newKey() {
        // DISPLAY new key dialog
        showNewKeyDlg();

        // GET key name from dialog

        // CALL add key function on project with key name
    }

    /**
     Displays a dialog to let the user enter options about a new key, then
     it creates the new key and adds it to all ResBundles in the project.

     PRE: none
     POST: new key may be added to the root
    */
    public void showNewKeyDlg() {
        // DISPLAY new key dialog
    }


    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     */
    private PrefDlg prefDlg;

    /**
     * @supplierCardinality 1
     * @clientCardinality 1
     */
    private FindDlg searchDlg;

    /**
     * @supplierCardinality 1
     * @clientCardinality 1
     */
    Project project;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     */
    MainWindow mainWindow;

    static RBEdit rbedit;
}