package net.quartzware;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import java.net.*;
import java.io.File;
import java.util.*;


/**
 Displays dialog box containing warning message when removing a child bundle from a project.
 Allows the user to decide whether they really want to remove the child or not.
 @author Anthony Tomarchio
 @version 0.7.1
*/

// Version History
//     11/30/00 - Version 0.9   Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//     11/29/00 - Version 0.7.1 - Tony Johansen
//                  Added GUI pseudocode
//     11/20/00 - Version 0.6 - Anthony Tomarchio
//                Updated for internationalization
//     11/19/00 - version 0.5 Anthony Tomarchio
//                Inserted GUI code from Visual Cafe
//                Formatted Code for organization and readability
//                Added JavaDoc Code to top of class
//     11/17/00 - version 0.1 Anthony Tomarchio
//                Code created in Visual Cafe

public class RemoveBundleDlg extends CenterDlg {

    // GUI elements declaration
    ImgPanel panel = new ImgPanel();
    JButton yesButton = new JButton();
    JButton noButton = new JButton();
    JTextArea remBundleMsg = new JTextArea();

    //Internationalization variables
        Locale currentLocale = Locale.getDefault();
        ResourceBundle bundle = ResourceBundle.getBundle("RBEdit", currentLocale);

    /**
     Constructs and displays the Remove Bundle Dialog
     @param parent The value of this resource bundle's parent.

     PRE:  parent is not null.
     POST: The dialog is created.
    */

    public RemoveBundleDlg(Frame parent) {
        super(parent);

        // CALL functions to layout this dialog
        doTheLayout();

        // ADD Listeners
        addListeners();

        // DISPLAY the dialog
        pack();
        setVisible(true);
    }

    /**
     Default Remove Bundle Dialog Constructor

     PRE:  none
     POST: The Remove Bundle Dialog is created.
    */
    public RemoveBundleDlg() {
        this((Frame)null);
    }

    /**
     Constructs and Displays Remove Bundle Dialog
     @param sTitle The title of the child.

     PRE:  none
     POST: The Remove Bundle Dialog is created.
    */
    public RemoveBundleDlg(String sTitle) {
        this();
        setTitle(sTitle);
    }

    /**
     This function handles the layout of the Remove Bundle Dialog box

     PRE: attributes for this class have been set
     POST: GUI components have been added to the dialog and arranged appropriately.
    */
    private void doTheLayout() {
        //INIT Remove Bundle Dialog - created by Visual Cafe
        setTitle(bundle.getString("removeBundleDlg"));
        setResizable(false);
        setModal(true);
        getContentPane().setLayout(null);
        setSize(277,125);
        setVisible(false);

        //INIT image panel
        panel.add(new ImageIcon(RBEdit.class.getResource(bundle.getString("removeBundleImage"))).getImage());
        panel.setLayout(null);
        panel.setBounds(12,12,80,65);
        getContentPane().add(panel);

        //INIT yes button
        yesButton.setText(bundle.getString("yesButton"));
        yesButton.setBounds(24,84,100,25);
        getContentPane().add(yesButton);

        //INIT no button
        noButton.setText(bundle.getString("noButton"));
        noButton.setBounds(156,84,100,25);
        getContentPane().add(noButton);

        //INIT Warning Dialogs
        remBundleMsg.setText(bundle.getString("remBundleMsg"));
        remBundleMsg.setFont(new Font("Dialog", Font.BOLD, 12));
        remBundleMsg.setBounds(96,24,180,40);

        remBundleMsg.setOpaque(false);
        remBundleMsg.setEditable(false);
        remBundleMsg.setLineWrap(true);

        getContentPane().add(remBundleMsg);
    }

    /**
     This function adds actionListeners to buttons which need to be watched.

     PRE: all GUI components have been initialized.
     POST: Listeners added to all componenets that need one.
    */
    private void addListeners() {
        //INIT action listener
        yesButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });

        noButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });
    }

    /**
     This function sets the visible status of a dialog.
     @param b Wether the dialog is to be visible or not.

     PRE: all GUI components have been initialized.
     POST: Dialog is visable or not visable.
    */

    public void setVisible(boolean b) {
        //If b is true INIT default loction
        if (b) {
            setLocation(50, 50);
        }

        //INIT visible status.
        super.setVisible(b);
    }

}