package net.quartzware;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import java.util.*;
import java.net.*;
import java.io.*;


/**
 Displays dialog box containing warning message when renaming a key.
 @author Anthony Tomarchio
 @version 0.7.1
*/

// Version History
//     11/30/00 - Version 0.9   Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//     11/29/00 - Version 0.7 - Tony Johansen
//                  Added GUI pseudocode
//              - Anthony Tomarchio - Internationalized
//     11/19/00 - version 0.5 Anthony Tomarchio
//                Inserted GUI code from Visual Cafe
//                Formatted Code for organization and readability
//                Added JavaDoc Code to top of class
//     11/17/00 - version 0.1 Anthony Tomarchio
//                Code created in Visual Cafe

public class RenameKeyDlg extends CenterDlg {

    // GUI elements declaration
    ImgPanel renameKeyPanel = new ImgPanel();
    JTextArea renameKeyMsg = new JTextArea();
    JButton okButton = new JButton();
    JButton cancelButton = new JButton();

    //Internationalization variables
    Locale currentLocale = Locale.getDefault();
    ResourceBundle bundle = ResourceBundle.getBundle("RBEdit", currentLocale);

    /**
     Constructs and displays the Rename Key Dialog
     @param parent The value of this resource bundle's parent.

     PRE:  parent is not null.
     POST: The dialog is created.
    */
    public RenameKeyDlg(Frame parent) {
        super(parent);
        // CALL doTheLayout
        doTheLayout();

        // ADD Listeners
        addListeners();

        // DISPLAY the dialog
        pack();
        setVisible(true);
    }

    /**
     Default Rename Key Dialog Constructor

     PRE:  none
     POST: The Rename Key Dialog is created.
    */
    public RenameKeyDlg() {
        // CALL constructor which takes a frame.
        this((Frame)null);
    }


    /**
     Constructs and Displays Rename Key Dialog
     @param sTitle The title of the key.

     PRE:  none
     POST: The Rename key Dialog is created.
    */
    public RenameKeyDlg(String sTitle) {
        this();
        setTitle(sTitle);
    }

    /**
     This function handles the layout of the Rename Key dialog box

     PRE: attributes have been set
     POST: GUI components have been added to the dialog and arranged appropriately.
    */
    private void doTheLayout() {
        //INIT Rename Key Dialog - created by Visual Cafe
        setTitle(bundle.getString("renameKeyDlg"));
        setResizable(false);
        setModal(true);
        getContentPane().setLayout(null);
        setSize(315,132);

        renameKeyPanel.add(new ImageIcon(RBEdit.class.getResource(
            bundle.getString("renameKeyPanel"))).getImage());
        renameKeyPanel.setLayout(null);
        renameKeyPanel.setBounds(13,13,75,62);
        getContentPane().add(renameKeyPanel);

        renameKeyMsg.setText(bundle.getString("renameKeyMsg"));
        renameKeyMsg.setLineWrap(true);
        renameKeyMsg.setOpaque(false);
        renameKeyMsg.setEditable(false);
        renameKeyMsg.setFont(new Font("Dialog", Font.BOLD, 12));
        renameKeyMsg.setBounds(96,12,204,84);
        getContentPane().add(renameKeyMsg);

        //INIT Ok Button
        okButton.setText(bundle.getString("okButton"));
        okButton.setMnemonic((int)'O');
        getContentPane().add(okButton);
        okButton.setBounds(48,96,100,25);

        //INIT Cancel Button
        cancelButton.setText(bundle.getString("cancelButton"));
        cancelButton.setMnemonic((int)'C');
        cancelButton.setBounds(180,96,100,25);
        getContentPane().add(cancelButton);
    }

    /**
     This function adds actionListeners to buttons which need to be watched.

     PRE: all GUI components have been initialized.
     POST: Listeners added to all componenets that need one.
    */
    private void addListeners() {
        //INIT OK Button Listener
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });

        //INIT Cancel Button Listener
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });
    }

    /**
     This function sets the visible status of a dialog.
     @param b Wether the dialog is to be visible or not.

     PRE: all GUI components have been initialized.
     POST: Dialog is visable or not visable.
    */
    public void setVisible(boolean b) {
        //If b is true INIT default loction
        if (b) {
            setLocation(50, 50);
        }

        //INIT visible status.
        super.setVisible(b);
    }
}