/* Generated by Together */

package net.quartzware;

import java.lang.*;
import java.util.*;
import java.io.*;

/**
 Encapsulates the data and functions of a 'resource bundle'.
 @author Chris Perluss
 @version 0.7.1
*/

// Version History
//     11/30/00 - Version 0.9   Ben Leinweber and Chris Perluss
//                Checked for missing ADT Headers and pseudocode.
//                Converted tabs to spaces and fixed spacing errors
//     11/17/00 - Version 0.3 - Chris Perluss
//                Added PRE and POST conditions
//     11/15/00 - Version 0.2 - Tony Tomarchio
//                Added algorithm pseudocode
//     11/01/00 - Version 0.1 - Chris Perluss
//                Initial Creation. JavaDocs created.

public class ResBundle implements RBListener {
    /**
     * @supplierCardinality 0..*
     * @clientCardinality 1
     */
    private Vector entries = new Vector();

    private Vector rbListeners = new Vector();

    /** @link dependency */
    /*#Entry lnkEntry;*/
    private EditorWindow parent;
    String suffix;
    File file;

    boolean dirty = false;
    boolean looseFile;

    /**
     Default Constructor

     PRE:  none
     POST: This class is created
    */
    public ResBundle(String suffix, File file, boolean loose) {
        // INIT entries as a new Vector
        this.file = file;
        looseFile = loose;
        this.suffix = suffix;

        doSetup();
    }

    /**
     Constructs a ResBundle from the given array of entries
     @param entries an array of entries from which to create this ResBundle

     PRE: entries does not equal null
     POST: this class is created
    */
    public ResBundle(Entry[] entries, String suffix, File file, boolean loose) {
        // INIT this.entries with new Vector
        looseFile = loose;

        // FOR ndx from 0 to length of entries INCREMENT 1 LOOP
        //     ADD entries[ndx] to this.entries
        for (int ndx = 0; ndx < entries.length; ndx++)
            this.entries.add(entries[ndx]);

        //INIT this.suffix with suffix
        this.suffix = suffix;
        this.file = file;

        doSetup();
    }

    /**
     Constructs a ResBundle from the given vector of entries
     @param entries a vector of entries from which to create this ResBundle

     PRE:  entries does not equal null
     POST: this class is created
    */
    public ResBundle(String suffix, File file, boolean loose, boolean unique) {
        looseFile = loose;

        //INIT this.suffix with suffix
        this.suffix = suffix;
        this.file = file;

        doSetup();
    }

    /**
     Perform setup that is required regardless of the constructor that was used.

     PRE:  none
     POST: none
    */
    public void doSetup() {
        ResBundle bundle;

        // SET bundle to the parent bundle
        if (suffix.equals("")) {
            return;
        } else if (suffix.indexOf('_') < 0) {
            bundle = RBEdit.getRBEdit().getProject().getRootBundle();
        } else {
            bundle = RBEdit.getRBEdit().getProject().getBundle(
                suffix.substring(0, suffix.lastIndexOf('_')));
        }

        bundle.addRBListener(this);
    }

    /**
     Returns suffix of this ResBundle
     @return the suffix

     PRE:  none
     POST: suffix is returned
    */
    public String getSuffix() {
        return suffix;
    }

    /**
     Sets the suffix of this ResBundle
     @param sfx the suffix to set

     PRE:  sfx is not null
     POST: suffix is set
    */
    public void setSuffix(String sfx) {
        suffix = sfx;
    }

    /**
     Gets the filename of this ResBundle
     @return the filename of this ResBundle

     PRE:  none
     POST: none
    */
    public File getFile() {
        return file;
    }

    /**
     Sets the filename of this ResBundle
     @param the new filename of this ResBundle

     PRE:  file is not null
     POST: none
    */
    public void setFile(File file) {
        this.file = file;
    }

    /**
     Returns whether this editor window has been modified
     @returns whether this editor window has been modified

     PRE: none
     POST: none
    */
    public boolean isDirty() {
        return dirty;
    }

    /**
     Sets whether this editor window has been modified
     @param whether this editor window has been modified

     PRE: none
     POST: if set to true, this bundle will be saved next time save is pressed
    */
    public void setDirty(boolean modified) {
        dirty = modified;
    }

    /**
     Returns whether this EditorWindow represents a loose file.

     PRE: none
     POST: none
    */
    public boolean isLooseFile() {
        return looseFile;
    }

    /**
     Sets whether this EditorWindow represents a loose file.

     PRE: none
     POST: none
    */
    public void setLooseFile(boolean loose) {
        looseFile = loose;
    }

    /**
     Returns all entries held by this ResBundle
     @return the Entries

     PRE:  key is not equal null
     POST: the Entry which contains key is returned if it exists, else null is returned
    */
    public Entry getParentEntry(String key) {
       Vector bundle;

        // SET bundle to the parent bundle
        if (suffix.equals("")) {
            return null;
        } else if (suffix.indexOf('_') < 0) {
            bundle = RBEdit.getRBEdit().getProject().getRootBundle().
                getAllEntries();
        } else {
              bundle = RBEdit.getRBEdit().getProject().getBundle(
                suffix.substring(0, suffix.lastIndexOf('_'))).getAllEntries();
        }

        // SEARCH bundle for key
        //     IF found return entry
        //     ELSE RETURN null
        for (int ndx = 0; ndx < bundle.size(); ndx++) {
            if (((Entry)bundle.elementAt(ndx)).getKey().equals(key)) {
                return (Entry)bundle.elementAt(ndx);
            }
		}

        // IF key not found THEN
        //    RETURN NULL
        return null;
    }

    /**
     Adds a listener.
     @param list the listener to add.

     PRE:  list != null
     POST: list may receive update events.
    */
    public void addRBListener(RBListener list) {
        rbListeners.addElement(list);
    }

    /**
     Removes a listener.
     @param list the listener to remove.

     PRE:  list != null
     POST: list will no longer receive update notices
    */
    public void remRBListener(RBListener list) {
        rbListeners.removeElement(list);
    }

    /**
     fires an update to all listeners.

     PRE:  none
     POST: listeners will be notified
    */
    public void fireEntryAdded(Entry ent, int index) {
        RBEvent rbe = new RBEvent(this, ent, index);

        for (int ndx = 0; ndx < rbListeners.size(); ndx++) {
           ((RBListener)rbListeners.elementAt(ndx)).entryAdded(rbe);
       }
    }

    /**
     fires an update to all listeners.

     PRE:  none
     POST: listeners will be notified
    */
    public void fireEntryInserted(Entry ent, int index) {
        RBEvent rbe = new RBEvent(this, ent, index);

        for (int ndx = 0; ndx < rbListeners.size(); ndx++) {
           ((RBListener)rbListeners.elementAt(ndx)).entryInserted(rbe);
       }
    }

    /**
     fires an update to all listeners.

     PRE:  none
     POST: listeners will be notified
    */
    public void fireEntryRemoved(Entry ent, int index) {
        RBEvent rbe = new RBEvent(this, ent, index);

        for (int ndx = 0; ndx < rbListeners.size(); ndx++) {
           ((RBListener)rbListeners.elementAt(ndx)).entryRemoved(rbe);
       }
    }

    /**
     fires an update to all listeners.

     PRE:  none
     POST: listeners will be notified
    */
    public void fireEntryChanged(Entry ent, int index) {
        RBEvent rbe = new RBEvent(this, ent, index);

        for (int ndx = 0; ndx < rbListeners.size(); ndx++) {
           ((RBListener)rbListeners.elementAt(ndx)).entryChanged(rbe);
       }
    }

    /**
     Returns the number of listeners

     PRE:  none
     POST: none
    */
    public int getRBListenerCount() {
        return rbListeners.size();
    }

    /**
     Returns all entries held by this ResBundle
     @return the Entries

     PRE:  none
     POST: all entries are returned
    */
    public Vector getAllEntries() {
        return entries;
    }

    /**
     Returns the nth entry held by this ResBundle
     @param idx the index of the entry to return
     @return an Entry

     PRE:  idx is in the bounds of the array.
     POST: the entry is returned
    */
    public Entry getEntry(int idx) {
        return (Entry)entries.elementAt(idx);
    }

    /**
     Removes the entry corresponding to 'key' from this resbundle.
     @param entries an array of entries from which to create this ResBundle
     @return the index of the removed bundle.

     PRE: key is not equal to null
     POST: if entry corresponding to entry is found it is removed.
    */
    public int removeEntry(String key) {
        // SEARCH bundle for key
        //     IF found remove entry
        for (int ndx = 0; ndx < entries.size(); ndx++) {
            if (((Entry)entries.elementAt(ndx)).getKey().equals(key)) {
                Entry ent = (Entry)entries.elementAt(ndx);
                entries.removeElementAt(ndx);
                fireEntryRemoved(ent, ndx);
                return ndx;
            }
        }
        return -1;
    }

    /**
     Tell this EditorWindow who its parent is.
     @param ew The parent

     PRE:  none
     POST: this class has a reference to its parent.
    */
    public void setEditorWindow(EditorWindow ew) {
        parent = ew;
    }

    /**
     Adds a new key to this ResBundle
     @param key the key to add

     PRE:  e does not equal null
     POST: the entry is added to this resource bundle.
    */
    public void addEntry(Entry ent) {
        // ADD entry
        entries.add(ent);
        fireEntryAdded(ent, entries.size()-1);
    }

    /**
     Inserts a key to this ResBundle
     @param key the key to add

     PRE:  ent does not equal null
     POST: the entry is added to this resource bundle.
    */
    public void insertEntryAt(Entry ent, int index) {
        entries.insertElementAt(ent, index);
        fireEntryInserted(ent, index);
    }

    public void entryAdded(RBEvent rbe) {}
    public void entryInserted(RBEvent rbe) {}
    public void entryRemoved(RBEvent rbe) {}

    /**
     Called when one of my parent's entries was changed.  If I don't overwrite
     that particular value (i.e., i am unique), then i ignore this event.  Else,
     I inherit this value from my parent, and since he changed, i must change to
     match.  After changing, I then inform my children that _I_ changed.
    */
    public void entryChanged(RBEvent rbe) {
        int ndx;

        // Avoid infinite loop, so return if src is my child
        if (!suffix.startsWith(rbe.getBundle().getSuffix())) {
            return;
        }

        // Find the affected Entry
        for (ndx = 0; ndx < entries.size(); ndx++) {
            if (((Entry)entries.elementAt(ndx)).getKey().equals(rbe.getKey())) {
                break;
            }
        }

        // Entry not found
        if (ndx == entries.size()) {
            return;
        }

        Entry ent = (Entry)entries.elementAt(ndx);

        // If the Entry is unique (not inherited), then the parent's change does
        if (ent.isUnique()) {                                   // not affect us
            return;
        }

        // Change our value
        ent.setValue(rbe.getEntry().getValue());

        // Inform my children that my Entry changed
        fireEntryChanged(ent, ndx);
    }

    public String toString() {
        String str;

        if (file != null) {
            str = file.getName();
        } else {
            str = "NewBundle";
        }

        if (suffix.equals("")) {
            str = str + " : root";
        } else {
            str = str + " : " + suffix;
        }

        return str;
    }
}
